"""CLI interface for pypreset."""

import contextlib
import logging
from pathlib import Path
from typing import TYPE_CHECKING, Annotated

import typer
from rich import print as rprint
from rich.console import Console
from rich.panel import Panel
from rich.table import Table

from pypreset.augment_generator import augment_project
from pypreset.generator import generate_project
from pypreset.interactive_prompts import run_auto_session, run_interactive_session
from pypreset.models import (
    CreationPackageManager,
    LayoutStyle,
    OverrideOptions,
    TypeChecker,
    TypingLevel,
)
from pypreset.preset_loader import (
    build_project_config,
    list_available_presets,
    load_preset,
    resolve_preset_chain,
)
from pypreset.project_analyzer import analyze_project
from pypreset.user_config import (
    get_config_path,
    get_default_config_template,
    load_user_config,
    save_user_config,
)
from pypreset.validator import validate_project, validate_with_poetry
from pypreset.versioning import VersioningAssistant, VersioningError

if TYPE_CHECKING:
    from pypreset.augment_generator import AugmentResult
    from pypreset.interactive_prompts import AugmentConfig
    from pypreset.models import ProjectConfig

app = typer.Typer(
    name="pypreset",
    help="A meta-tool for setting up Poetry-based Python projects with configurable presets.",
    no_args_is_help=True,
)

version_app = typer.Typer(
    name="version",
    help="Versioning assistance commands.",
    no_args_is_help=True,
)

app.add_typer(version_app, name="version")

config_app = typer.Typer(
    name="config",
    help="Manage user-level default preferences.",
    no_args_is_help=True,
)
app.add_typer(config_app, name="config")

console = Console()

# Configure logging
logging.basicConfig(
    level=logging.INFO,
    format="%(levelname)s: %(message)s",
)
logger = logging.getLogger(__name__)


def _create_versioning_assistant(project_dir: Path) -> VersioningAssistant:
    return VersioningAssistant(project_dir)


@app.command("create")
def create_project(
    name: Annotated[str, typer.Argument(help="Name of the project to create")],
    preset: Annotated[
        str, typer.Option("--preset", "-p", help="Preset to use for project setup")
    ] = "empty-package",
    output_dir: Annotated[
        Path, typer.Option("--output", "-o", help="Output directory for the project")
    ] = Path("."),
    config: Annotated[
        Path | None, typer.Option("--config", "-c", help="Custom preset config file")
    ] = None,
    testing: Annotated[
        bool | None, typer.Option("--testing/--no-testing", help="Enable/disable testing")
    ] = None,
    formatting: Annotated[
        bool | None,
        typer.Option("--formatting/--no-formatting", help="Enable/disable formatting"),
    ] = None,
    python_version: Annotated[
        str | None, typer.Option("--python-version", help="Python version (e.g., 3.11)")
    ] = None,
    typing_level: Annotated[
        TypingLevel | None,
        typer.Option("--typing", help="Typing strictness level"),
    ] = None,
    layout: Annotated[
        LayoutStyle | None,
        typer.Option("--layout", help="Project layout style (src or flat)"),
    ] = None,
    radon: Annotated[
        bool | None,
        typer.Option("--radon/--no-radon", help="Enable radon complexity checking"),
    ] = None,
    pre_commit: Annotated[
        bool | None,
        typer.Option("--pre-commit/--no-pre-commit", help="Generate pre-commit hooks config"),
    ] = None,
    version_bumping: Annotated[
        bool | None,
        typer.Option(
            "--bump-my-version/--no-bump-my-version",
            help="Include bump-my-version for version management",
        ),
    ] = None,
    type_checker: Annotated[
        TypeChecker | None,
        typer.Option("--type-checker", help="Type checker (mypy, pyright, or ty)"),
    ] = None,
    package_manager: Annotated[
        CreationPackageManager | None,
        typer.Option("--package-manager", help="Package manager (poetry or uv)"),
    ] = None,
    docker: Annotated[
        bool | None,
        typer.Option("--docker/--no-docker", help="Generate Dockerfile and .dockerignore"),
    ] = None,
    devcontainer: Annotated[
        bool | None,
        typer.Option(
            "--devcontainer/--no-devcontainer", help="Generate .devcontainer/ configuration"
        ),
    ] = None,
    extra_package: Annotated[
        list[str] | None,
        typer.Option("--extra-package", "-e", help="Additional packages to install"),
    ] = None,
    extra_dev_package: Annotated[
        list[str] | None,
        typer.Option("--extra-dev-package", "-d", help="Additional dev packages"),
    ] = None,
    init_git: Annotated[
        bool, typer.Option("--git/--no-git", help="Initialize git repository")
    ] = True,
    install: Annotated[
        bool, typer.Option("--install/--no-install", help="Run dependency install")
    ] = False,
    dry_run: Annotated[
        bool,
        typer.Option("--dry-run", help="Preview what would be created without generating anything"),
    ] = False,
    verbose: Annotated[bool, typer.Option("--verbose", "-v", help="Enable verbose output")] = False,
) -> None:
    """Create a new Python project from a preset."""
    if verbose:
        logging.getLogger().setLevel(logging.DEBUG)

    # Build override options
    overrides = OverrideOptions(
        testing_enabled=testing,
        formatting_enabled=formatting,
        radon_enabled=radon,
        pre_commit_enabled=pre_commit,
        version_bumping_enabled=version_bumping,
        type_checker=type_checker,
        python_version=python_version,
        typing_level=typing_level,
        layout=layout,
        package_manager=package_manager,
        docker_enabled=docker,
        devcontainer_enabled=devcontainer,
        extra_packages=extra_package or [],
        extra_dev_packages=extra_dev_package or [],
    )

    try:
        # Build project configuration
        project_config = build_project_config(
            project_name=name,
            preset_name=preset,
            overrides=overrides,
            custom_preset_path=config,
        )

        if dry_run:
            _display_dry_run(name, preset, output_dir.absolute(), project_config, init_git, install)
            return

        rprint(f"[blue]Creating project '{name}' with preset '{preset}'...[/blue]")

        # Generate the project
        project_dir = generate_project(
            config=project_config,
            output_dir=output_dir.absolute(),
            initialize_git=init_git,
            install_dependencies=install,
        )

        # Validate the generated project
        is_valid, results = validate_project(project_dir)

        if is_valid:
            rprint(
                Panel.fit(
                    f"[green]✓ Project '{name}' created successfully![/green]\n\n"
                    f"Location: [cyan]{project_dir}[/cyan]\n\n"
                    f"[dim]Next steps:[/dim]\n"
                    f"  cd {name}\n"
                    f"  poetry install\n"
                    f"  poetry run pytest",
                    title="Success",
                )
            )
        else:
            rprint("[yellow]Project created with warnings:[/yellow]")
            for result in results:
                if not result.passed:
                    rprint(f"  [yellow]⚠ {result.message}[/yellow]")

    except ValueError as e:
        rprint(f"[red]Error: {e}[/red]")
        raise typer.Exit(1) from None
    except Exception as e:
        rprint(f"[red]Unexpected error: {e}[/red]")
        if verbose:
            import traceback

            traceback.print_exc()
        raise typer.Exit(1) from None


def _display_dry_run(
    name: str,
    preset: str,
    output_dir: Path,
    config: "ProjectConfig",
    init_git: bool,
    install: bool,
) -> None:
    """Display a dry-run summary of what would be created."""
    from pypreset.models import CreationPackageManager, LayoutStyle
    from pypreset.template_engine import render_path

    package_name = name.replace("-", "_")
    is_src = config.layout == LayoutStyle.SRC
    is_uv = config.package_manager == CreationPackageManager.UV
    project_dir = output_dir / name
    context = {"project": {"name": name, "package_name": package_name}}

    # --- Header ---
    rprint(
        Panel.fit(
            "[bold]Dry run[/bold] — nothing will be created",
            title=f"pypreset create {name} --preset {preset}",
            border_style="yellow",
        )
    )

    # --- Project overview ---
    overview = Table(title="Project Overview", show_header=False, box=None, padding=(0, 2))
    overview.add_column(style="cyan")
    overview.add_column()
    overview.add_row("Location", str(project_dir))
    overview.add_row("Preset", preset)
    overview.add_row("Package manager", config.package_manager.value)
    overview.add_row("Layout", config.layout.value)
    overview.add_row("Python", config.metadata.python_version)
    overview.add_row(
        "Typing", f"{config.typing_level.value} ({config.formatting.type_checker.value})"
    )
    overview.add_row(
        "Testing", config.testing.framework.value if config.testing.enabled else "disabled"
    )
    overview.add_row(
        "Formatting", config.formatting.tool.value if config.formatting.enabled else "disabled"
    )
    overview.add_row("Git init", "yes" if init_git else "no")
    overview.add_row("Install deps", "yes" if install else "no")
    console.print(overview)
    rprint()

    # --- Feature flags ---
    flags: list[tuple[str, bool]] = [
        ("Radon complexity", config.formatting.radon),
        ("Pre-commit hooks", config.formatting.pre_commit),
        ("bump-my-version", config.formatting.version_bumping),
        ("Dependabot", config.dependabot.enabled),
        ("Coverage", config.testing.coverage),
        ("Docker", config.docker.enabled),
        ("Devcontainer", config.docker.devcontainer),
    ]
    active = [name for name, enabled in flags if enabled]
    if active:
        rprint(f"[cyan]Extras:[/cyan] {', '.join(active)}")
        rprint()

    # --- Directory tree ---
    tree_lines: list[str] = []
    pkg_prefix = f"src/{package_name}" if is_src else package_name

    tree_lines.append(f"{name}/")
    tree_lines.append(f"  {pkg_prefix}/")
    tree_lines.append("    __init__.py")

    # Preset-defined directories
    for dir_path in config.structure.directories:
        rendered = render_path(dir_path, context)
        tree_lines.append(f"  {rendered}/")

    # Preset-defined files
    for file_def in config.structure.files:
        rendered = render_path(file_def.path, context)
        tree_lines.append(f"  {rendered}")

    if config.testing.enabled:
        tree_lines.append("  tests/")
        tree_lines.append("    __init__.py")
        tree_lines.append("    test_basic.py")

    pyproject_tmpl = "pyproject_uv.toml" if is_uv else "pyproject.toml"
    tree_lines.append(f"  {pyproject_tmpl.replace('_uv', '')} [dim]({pyproject_tmpl}.j2)[/dim]")
    tree_lines.append("  README.md")
    tree_lines.append("  .gitignore")

    if config.testing.enabled or config.formatting.enabled:
        ci_tmpl = "github_ci_uv.yaml" if is_uv else "github_ci.yaml"
        tree_lines.append(f"  .github/workflows/ci.yaml [dim]({ci_tmpl}.j2)[/dim]")

    if config.dependabot.enabled:
        tree_lines.append("  .github/dependabot.yml")

    if config.formatting.pre_commit:
        tree_lines.append("  .pre-commit-config.yaml")

    if config.docker.enabled:
        tree_lines.append("  Dockerfile")
        tree_lines.append("  .dockerignore")

    if config.docker.devcontainer:
        tree_lines.append("  .devcontainer/")
        tree_lines.append("    devcontainer.json")

    rprint(Panel("\n".join(tree_lines), title="Project Structure", border_style="green"))

    # --- Dependencies ---
    if config.dependencies.main or config.dependencies.dev:
        dep_table = Table(title="Dependencies")
        dep_table.add_column("Package", style="white")
        dep_table.add_column("Group", style="dim")

        for pkg in config.dependencies.main:
            dep_table.add_row(pkg, "main")
        for pkg in config.dependencies.dev:
            dep_table.add_row(pkg, "dev")

        console.print(dep_table)

    # --- Entry points ---
    if config.entry_points:
        rprint("\n[cyan]Entry points:[/cyan]")
        for ep in config.entry_points:
            rprint(f"  {ep.name} = {ep.module}")


@app.command("list-presets")
def list_presets_cmd() -> None:
    """List all available presets."""
    presets = list_available_presets()

    if not presets:
        rprint("[yellow]No presets found.[/yellow]")
        return

    table = Table(title="Available Presets")
    table.add_column("Name", style="cyan")
    table.add_column("Description")

    for name, description in presets:
        table.add_row(name, description)

    console.print(table)


@app.command("show-preset")
def show_preset_cmd(
    preset_name: Annotated[str, typer.Argument(help="Name of the preset to show")],
) -> None:
    """Show details of a specific preset."""
    try:
        preset = load_preset(preset_name)
        resolved = resolve_preset_chain(preset)

        rprint(
            Panel.fit(
                f"[bold]{preset.name}[/bold]\n\n[dim]{preset.description}[/dim]",
                title="Preset Info",
            )
        )

        if preset.base:
            rprint(f"\n[cyan]Extends:[/cyan] {preset.base}")

        # Show dependencies
        deps = resolved.get("dependencies", {})
        if deps.get("main"):
            rprint("\n[cyan]Dependencies:[/cyan]")
            for pkg in deps["main"]:
                rprint(f"  • {pkg}")

        if deps.get("dev"):
            rprint("\n[cyan]Dev Dependencies:[/cyan]")
            for pkg in deps["dev"]:
                rprint(f"  • {pkg}")

        # Show structure
        structure = resolved.get("structure", {})
        if structure.get("directories"):
            rprint("\n[cyan]Directories:[/cyan]")
            for dir_path in structure["directories"]:
                rprint(f"  📁 {dir_path}")

        if structure.get("files"):
            rprint("\n[cyan]Files:[/cyan]")
            for file_info in structure["files"]:
                path = file_info.get("path", "unknown")
                rprint(f"  📄 {path}")

        # Show entry points
        entry_points = resolved.get("entry_points", [])
        if entry_points:
            rprint("\n[cyan]Entry Points:[/cyan]")
            for ep in entry_points:
                rprint(f"  • {ep['name']} → {ep['module']}")

    except ValueError as e:
        rprint(f"[red]Error: {e}[/red]")
        raise typer.Exit(1) from None


@app.command("validate")
def validate_cmd(
    project_dir: Annotated[Path, typer.Argument(help="Path to the project to validate")] = Path(
        "."
    ),
    poetry_check: Annotated[
        bool, typer.Option("--poetry/--no-poetry", help="Also run poetry check")
    ] = False,
) -> None:
    """Validate an existing project."""
    project_path = project_dir.absolute()

    if not project_path.exists():
        rprint(f"[red]Error: Directory '{project_path}' does not exist[/red]")
        raise typer.Exit(1)

    rprint(f"[blue]Validating project at {project_path}...[/blue]\n")

    is_valid, results = validate_project(project_path)

    for result in results:
        if result.passed:
            rprint(f"  [green]✓[/green] {result.message}")
        else:
            rprint(f"  [red]✗[/red] {result.message}")

    if poetry_check:
        rprint("\n[blue]Running poetry check...[/blue]")
        poetry_result = validate_with_poetry(project_path)
        if poetry_result.passed:
            rprint(f"  [green]✓[/green] {poetry_result.message}")
        else:
            rprint(f"  [red]✗[/red] {poetry_result.message}")
            if poetry_result.details:
                rprint(f"    [dim]{poetry_result.details}[/dim]")

    if is_valid:
        rprint("\n[green]All validations passed![/green]")
    else:
        rprint("\n[red]Some validations failed.[/red]")
        raise typer.Exit(1)


def _apply_component_overrides(
    config: "AugmentConfig",
    *,
    test_workflow: bool | None,
    lint_workflow: bool | None,
    dependabot: bool | None,
    tests_dir: bool | None,
    gitignore: bool | None,
    pypi_publish: bool | None,
    dockerfile_flag: bool | None = None,
    devcontainer_flag: bool | None = None,
) -> None:
    """Apply CLI component overrides to an AugmentConfig in place."""
    overrides: list[tuple[str, bool | None]] = [
        ("generate_test_workflow", test_workflow),
        ("generate_lint_workflow", lint_workflow),
        ("generate_dependabot", dependabot),
        ("generate_tests_dir", tests_dir),
        ("generate_gitignore", gitignore),
        ("generate_pypi_publish", pypi_publish),
        ("generate_dockerfile", dockerfile_flag),
        ("generate_devcontainer", devcontainer_flag),
    ]
    for attr, value in overrides:
        if value is not None:
            setattr(config, attr, value)


def _display_augment_result(result: "AugmentResult") -> None:
    """Print augment operation results and raise on errors."""
    if result.files_created:
        rprint("\n[green]✓ Created files:[/green]")
        for file in result.files_created:
            status = "[yellow](overwritten)[/yellow]" if file.overwritten else ""
            rprint(f"  • {file.path} {status}")

    if result.files_skipped:
        rprint("\n[yellow]⚠ Skipped files (already exist):[/yellow]")
        for path in result.files_skipped:
            rprint(f"  • {path}")

    if result.errors:
        rprint("\n[red]✗ Errors:[/red]")
        for error in result.errors:
            rprint(f"  • {error}")
        raise typer.Exit(1)

    if result.success:
        rprint(
            Panel.fit(
                "[green]✓ Project augmented successfully![/green]\n\n"
                f"[dim]Generated {len(result.files_created)} file(s)[/dim]",
                title="Success",
            )
        )
    else:
        rprint("\n[yellow]Project augmented with warnings.[/yellow]")


@app.command("augment")
def augment_cmd(
    project_dir: Annotated[Path, typer.Argument(help="Path to the project to augment")] = Path("."),
    interactive: Annotated[
        bool,
        typer.Option(
            "--interactive/--auto",
            "-i/-a",
            help="Interactive mode for missing values (default) or auto-detect everything",
        ),
    ] = True,
    force: Annotated[
        bool,
        typer.Option("--force", "-f", help="Overwrite existing files"),
    ] = False,
    test_workflow: Annotated[
        bool | None,
        typer.Option("--test-workflow/--no-test-workflow", help="Generate test workflow"),
    ] = None,
    lint_workflow: Annotated[
        bool | None,
        typer.Option("--lint-workflow/--no-lint-workflow", help="Generate lint workflow"),
    ] = None,
    dependabot: Annotated[
        bool | None,
        typer.Option("--dependabot/--no-dependabot", help="Generate dependabot.yml"),
    ] = None,
    tests_dir: Annotated[
        bool | None,
        typer.Option("--tests/--no-tests", help="Generate tests directory"),
    ] = None,
    gitignore: Annotated[
        bool | None,
        typer.Option("--gitignore/--no-gitignore", help="Generate .gitignore file"),
    ] = None,
    pypi_publish: Annotated[
        bool | None,
        typer.Option("--pypi-publish/--no-pypi-publish", help="Generate PyPI publish workflow"),
    ] = None,
    dockerfile: Annotated[
        bool | None,
        typer.Option("--dockerfile/--no-dockerfile", help="Generate Dockerfile and .dockerignore"),
    ] = None,
    devcontainer_flag: Annotated[
        bool | None,
        typer.Option(
            "--devcontainer/--no-devcontainer", help="Generate .devcontainer/ configuration"
        ),
    ] = None,
    verbose: Annotated[bool, typer.Option("--verbose", "-v", help="Enable verbose output")] = False,
) -> None:
    """Augment an existing project with workflows, tests, and configuration.

    Analyzes your pyproject.toml and project structure to automatically generate:
    - GitHub Actions test workflow
    - GitHub Actions lint workflow
    - Dependabot configuration
    - Tests directory with template tests
    - .gitignore file

    If information cannot be reliably detected, an interactive prompt will ask
    for the missing details. Use --auto to skip prompts and use detected/default values.

    Examples:
        pypreset augment                    # Augment current directory
        pypreset augment ./my-project       # Augment specific project
        pypreset augment --auto             # Auto-detect everything, no prompts
        pypreset augment --force            # Overwrite existing files
        pypreset augment --test-workflow    # Only generate test workflow
    """
    if verbose:
        logging.getLogger().setLevel(logging.DEBUG)

    project_path = project_dir.absolute()

    if not project_path.exists():
        rprint(f"[red]Error: Directory '{project_path}' does not exist[/red]")
        raise typer.Exit(1)

    pyproject_path = project_path / "pyproject.toml"
    if not pyproject_path.exists():
        rprint(f"[red]Error: No pyproject.toml found in '{project_path}'[/red]")
        rprint(
            "[dim]The augment command requires an existing Python project with pyproject.toml[/dim]"
        )
        raise typer.Exit(1)

    try:
        # Analyze the project
        rprint(f"[blue]🔍 Analyzing project at {project_path}...[/blue]")
        analysis = analyze_project(project_path)

        # Build augment configuration
        if interactive:
            config = run_interactive_session(analysis)
        else:
            config = run_auto_session(analysis)
            rprint("\n[cyan]📋 Auto-detected configuration:[/cyan]")
            rprint(f"  • Project: [bold]{config.project_name}[/bold]")
            rprint(f"  • Python: {config.python_version}")
            rprint(f"  • Package Manager: {config.package_manager.value}")
            rprint(f"  • Test Framework: {config.test_framework.value}")
            rprint(f"  • Linter: {config.linter.value}")
            rprint(f"  • Type Checker: {config.type_checker.value}")

        _apply_component_overrides(
            config,
            test_workflow=test_workflow,
            lint_workflow=lint_workflow,
            dependabot=dependabot,
            tests_dir=tests_dir,
            gitignore=gitignore,
            pypi_publish=pypi_publish,
            dockerfile_flag=dockerfile,
            devcontainer_flag=devcontainer_flag,
        )

        if not any(
            [
                config.generate_test_workflow,
                config.generate_lint_workflow,
                config.generate_dependabot,
                config.generate_tests_dir,
                config.generate_gitignore,
                config.generate_pypi_publish,
                config.generate_dockerfile,
                config.generate_devcontainer,
            ]
        ):
            rprint("[yellow]No components selected for generation.[/yellow]")
            raise typer.Exit(0)

        rprint("\n[blue]🛠️  Generating components...[/blue]")
        result = augment_project(project_path, config, force=force)
        _display_augment_result(result)

    except KeyboardInterrupt:
        rprint("\n[yellow]Augment cancelled.[/yellow]")
        raise typer.Exit(1) from None
    except Exception as e:
        rprint(f"[red]Error: {e}[/red]")
        if verbose:
            import traceback

            traceback.print_exc()
        raise typer.Exit(1) from None


@version_app.command("release")
def release_cmd(
    bump: Annotated[
        str,
        typer.Option(
            "--bump",
            "-b",
            help="Version bump (patch, minor, major, prerelease, etc.)",
        ),
    ] = "patch",
    project_dir: Annotated[
        Path, typer.Option("--path", "-p", help="Path to the project root")
    ] = Path("."),
) -> None:
    """Bump version, commit, tag, push, and create a GitHub release."""
    project_path = project_dir.absolute()
    try:
        rprint(f"[blue]🚀 Releasing with bump '{bump}'...[/blue]")
        assistant = _create_versioning_assistant(project_path)
        version = assistant.release(bump)
        rprint(
            Panel.fit(
                f"[green]✓ Release v{version} created.[/green]",
                title="Release",
            )
        )
    except VersioningError as e:
        rprint(f"[red]Error: {e}[/red]")
        raise typer.Exit(1) from None


@version_app.command("release-version")
def release_version_cmd(
    version: Annotated[str, typer.Argument(help="Explicit version to release")],
    project_dir: Annotated[
        Path, typer.Option("--path", "-p", help="Path to the project root")
    ] = Path("."),
) -> None:
    """Use an explicit version, then commit, tag, push, and release."""
    project_path = project_dir.absolute()
    try:
        rprint(f"[blue]🚀 Releasing version '{version}'...[/blue]")
        assistant = _create_versioning_assistant(project_path)
        normalized = assistant.release_version(version)
        rprint(
            Panel.fit(
                f"[green]✓ Release v{normalized} created.[/green]",
                title="Release",
            )
        )
    except VersioningError as e:
        rprint(f"[red]Error: {e}[/red]")
        raise typer.Exit(1) from None


@version_app.command("rerun")
def rerun_cmd(
    version: Annotated[str, typer.Argument(help="Version to re-tag and push")],
    project_dir: Annotated[
        Path, typer.Option("--path", "-p", help="Path to the project root")
    ] = Path("."),
) -> None:
    """Re-tag and push an existing version."""
    project_path = project_dir.absolute()
    try:
        rprint(f"[blue]🔁 Re-tagging and pushing '{version}'...[/blue]")
        assistant = _create_versioning_assistant(project_path)
        normalized = assistant.rerun(version)
        rprint(
            Panel.fit(
                f"[green]✓ Re-tagged v{normalized}.[/green]",
                title="Re-run",
            )
        )
    except VersioningError as e:
        rprint(f"[red]Error: {e}[/red]")
        raise typer.Exit(1) from None


@version_app.command("rerelease")
def rerelease_cmd(
    version: Annotated[str, typer.Argument(help="Version to delete and recreate")],
    project_dir: Annotated[
        Path, typer.Option("--path", "-p", help="Path to the project root")
    ] = Path("."),
) -> None:
    """Delete and recreate the GitHub release for a version."""
    project_path = project_dir.absolute()
    try:
        rprint(f"[blue]♻️  Recreating GitHub release '{version}'...[/blue]")
        assistant = _create_versioning_assistant(project_path)
        normalized = assistant.rerelease(version)
        rprint(
            Panel.fit(
                f"[green]✓ Recreated release v{normalized}.[/green]",
                title="Re-release",
            )
        )
    except VersioningError as e:
        rprint(f"[red]Error: {e}[/red]")
        raise typer.Exit(1) from None


@app.command("analyze")
def analyze_cmd(
    project_dir: Annotated[Path, typer.Argument(help="Path to the project to analyze")] = Path("."),
) -> None:
    """Analyze an existing project and display detected configuration.

    This is useful to preview what the augment command would detect
    before actually generating any files.
    """
    project_path = project_dir.absolute()

    if not project_path.exists():
        rprint(f"[red]Error: Directory '{project_path}' does not exist[/red]")
        raise typer.Exit(1)

    pyproject_path = project_path / "pyproject.toml"
    if not pyproject_path.exists():
        rprint(f"[red]Error: No pyproject.toml found in '{project_path}'[/red]")
        raise typer.Exit(1)

    rprint(f"[blue]🔍 Analyzing project at {project_path}...[/blue]\n")
    analysis = analyze_project(project_path)

    # Display analysis using the interactive prompter's display method
    from pypreset.interactive_prompts import InteractivePrompter

    prompter = InteractivePrompter(analysis)
    prompter.display_analysis_summary()


@config_app.command("show")
def config_show_cmd() -> None:
    """Show current user configuration."""
    config_path = get_config_path()
    user_cfg = load_user_config()

    if not user_cfg:
        rprint(f"[yellow]No user config found at {config_path}[/yellow]")
        rprint("[dim]Run 'pypreset config init' to create one.[/dim]")
        return

    rprint(f"[cyan]Config file:[/cyan] {config_path}\n")
    table = Table(title="User Defaults")
    table.add_column("Setting", style="cyan")
    table.add_column("Value")

    for key, value in user_cfg.items():
        table.add_row(key, str(value))

    console.print(table)


@config_app.command("init")
def config_init_cmd(
    force: Annotated[bool, typer.Option("--force", "-f", help="Overwrite existing config")] = False,
) -> None:
    """Create a default user configuration file."""
    config_path = get_config_path()

    if config_path.exists() and not force:
        rprint(f"[yellow]Config already exists at {config_path}[/yellow]")
        rprint("[dim]Use --force to overwrite.[/dim]")
        raise typer.Exit(1)

    template = get_default_config_template()
    saved_path = save_user_config(template)
    rprint(f"[green]Created default config at {saved_path}[/green]")
    rprint("[dim]Edit this file to customize your defaults.[/dim]")


@config_app.command("set")
def config_set_cmd(
    key: Annotated[str, typer.Argument(help="Config key to set")],
    value: Annotated[str, typer.Argument(help="Value to set")],
) -> None:
    """Set a single configuration value."""
    user_cfg = load_user_config()

    # Try to coerce numeric values
    coerced: str | int = value
    with contextlib.suppress(ValueError):
        coerced = int(value)

    user_cfg[key] = coerced
    save_user_config(user_cfg)
    rprint(f"[green]Set {key} = {coerced}[/green]")


@config_app.command("path")
def config_path_cmd() -> None:
    """Print the path to the user config file."""
    rprint(str(get_config_path()))


def main() -> None:
    """Main entry point."""
    app()


if __name__ == "__main__":
    main()



"""Template engine for rendering project files."""

import logging
from pathlib import Path
from typing import Any

from jinja2 import Environment, FileSystemLoader, select_autoescape

from pypreset.docker_utils import resolve_docker_base_image as _resolve_base_image
from pypreset.models import ProjectConfig

logger = logging.getLogger(__name__)


def get_templates_dir() -> Path:
    """Get the directory containing built-in templates."""
    return Path(__file__).parent / "templates"


def create_jinja_environment() -> Environment:
    """Create a Jinja2 environment with the templates directory."""
    templates_dir = get_templates_dir()
    return Environment(
        loader=FileSystemLoader(templates_dir),
        autoescape=select_autoescape(["html", "xml"]),
        trim_blocks=True,
        lstrip_blocks=True,
        keep_trailing_newline=True,
    )


def get_template_context(config: ProjectConfig) -> dict[str, Any]:
    """Build the template context from a project configuration."""
    # Convert package name to module name (replace hyphens with underscores)
    package_name = config.metadata.name.replace("-", "_")

    return {
        "project": {
            "name": config.metadata.name,
            "package_name": package_name,
            "version": config.metadata.version,
            "description": config.metadata.description,
            "authors": config.metadata.authors,
            "license": config.metadata.license,
            "readme": config.metadata.readme,
            "python_version": config.metadata.python_version,
            "keywords": config.metadata.keywords,
            "classifiers": config.metadata.classifiers,
        },
        "dependencies": {
            "main": config.dependencies.main,
            "dev": config.dependencies.dev,
            "optional": config.dependencies.optional,
        },
        "testing": {
            "enabled": config.testing.enabled,
            "framework": config.testing.framework.value,
            "coverage": config.testing.coverage,
        },
        "formatting": {
            "enabled": config.formatting.enabled,
            "tool": config.formatting.tool.value,
            "line_length": config.formatting.line_length,
            "radon": config.formatting.radon,
            "pre_commit": config.formatting.pre_commit,
            "version_bumping": config.formatting.version_bumping,
            "type_checker": config.formatting.type_checker.value,
        },
        "dependabot": {
            "enabled": config.dependabot.enabled,
            "schedule": config.dependabot.schedule,
            "open_pull_requests_limit": config.dependabot.open_pull_requests_limit,
        },
        "docker": {
            "enabled": config.docker.enabled,
            "base_image": _resolve_base_image(
                config.metadata.python_version, config.docker.base_image
            ),
            "devcontainer": config.docker.devcontainer,
        },
        "typing_level": config.typing_level.value,
        "layout": config.layout.value,
        "package_manager": config.package_manager.value,
        "entry_points": [{"name": ep.name, "module": ep.module} for ep in config.entry_points],
        "extras": config.extras,
    }


def render_template(env: Environment, template_name: str, context: dict[str, Any]) -> str:
    """Render a template with the given context."""
    template = env.get_template(template_name)
    return template.render(**context)


def render_content(content: str, context: dict[str, Any]) -> str:
    """Render inline content (not from a template file)."""
    env = Environment(
        trim_blocks=True,
        lstrip_blocks=True,
        keep_trailing_newline=True,
    )
    template = env.from_string(content)
    return template.render(**context)


def render_path(path_template: str, context: dict[str, Any]) -> str:
    """Render a path template (e.g., 'src/{{ project.package_name }}')."""
    env = Environment()
    template = env.from_string(path_template)
    return template.render(**context)



"""Versioning assistance utilities."""

from __future__ import annotations

import shutil
import subprocess
from dataclasses import dataclass
from typing import TYPE_CHECKING, Literal, Protocol

if TYPE_CHECKING:
    from pathlib import Path

ToolName = Literal["git", "poetry", "gh"]


class VersioningError(Exception):
    """Error raised for versioning failures."""


@dataclass(frozen=True)
class CommandFailure(VersioningError):
    """Error raised when a command fails."""

    command: list[str]
    returncode: int | None
    stdout: str
    stderr: str

    def __str__(self) -> str:
        command_str = " ".join(self.command)
        parts = [f"Command failed: {command_str}"]
        if self.returncode is not None:
            parts.append(f"Exit code: {self.returncode}")
        if self.stdout.strip():
            parts.append(f"Stdout: {self.stdout.strip()}")
        if self.stderr.strip():
            parts.append(f"Stderr: {self.stderr.strip()}")
        return "\n".join(parts)


class CommandRunner(Protocol):
    """Protocol for running shell commands."""

    def run(self, args: list[str], *, check: bool = True) -> subprocess.CompletedProcess[str]:
        """Run a command and return the completed process."""
        ...


class SubprocessRunner:
    """Subprocess-backed command runner."""

    def __init__(self, cwd: Path) -> None:
        self.cwd = cwd

    def run(self, args: list[str], *, check: bool = True) -> subprocess.CompletedProcess[str]:
        """Run a command with subprocess."""
        try:
            return subprocess.run(
                args,
                cwd=self.cwd,
                text=True,
                capture_output=True,
                check=check,
            )
        except subprocess.CalledProcessError as exc:
            raise CommandFailure(
                command=args,
                returncode=exc.returncode,
                stdout=exc.stdout or "",
                stderr=exc.stderr or "",
            ) from exc
        except FileNotFoundError as exc:
            raise CommandFailure(
                command=args,
                returncode=None,
                stdout="",
                stderr=f"{args[0]} not found",
            ) from exc


def _normalize_prefixed_value(value: str, key: Literal["bump", "version"]) -> str:
    normalized = value.strip()
    prefix = f"{key}="
    if normalized.startswith(prefix):
        normalized = normalized[len(prefix) :].strip()
    if not normalized:
        raise VersioningError(f"{key} cannot be empty")
    return normalized


def _check_required_tools(tools: list[ToolName]) -> None:
    missing = [tool for tool in tools if shutil.which(tool) is None]
    if missing:
        joined = ", ".join(missing)
        raise VersioningError(f"Missing required tools: {joined}")


def _tag_name(version: str) -> str:
    return f"v{version}"


class VersioningAssistant:
    """Implements versioning workflows inspired by the project's Justfile."""

    def __init__(
        self,
        project_dir: Path,
        runner: CommandRunner | None = None,
        *,
        preflight: bool = True,
    ) -> None:
        self.project_dir = project_dir
        self.runner = runner or SubprocessRunner(project_dir)
        if preflight:
            self._preflight()

    def release(self, bump: str) -> str:
        """Bump version, commit, tag, push, and create a GitHub release."""
        normalized = _normalize_prefixed_value(bump, "bump")
        self._run_checked(["poetry", "version", normalized])
        version = self._read_version()
        self._release(version)
        return version

    def release_version(self, version: str) -> str:
        """Use an explicit version, then commit, tag, push, and release."""
        normalized = _normalize_prefixed_value(version, "version")
        self._run_checked(["poetry", "version", normalized])
        self._release(normalized)
        return normalized

    def rerun(self, version: str) -> str:
        """Re-tag and push an existing version."""
        normalized = _normalize_prefixed_value(version, "version")
        tag = _tag_name(normalized)
        self._run_checked(["git", "push"])
        self._run_allowed_failure(["git", "tag", "-d", tag])
        self._run_allowed_failure(["git", "push", "--delete", "origin", tag])
        self._run_checked(["git", "tag", tag])
        self._run_checked(["git", "push", "origin", tag])
        return normalized

    def rerelease(self, version: str) -> str:
        """Delete and recreate a GitHub release for a version."""
        normalized = _normalize_prefixed_value(version, "version")
        tag = _tag_name(normalized)
        self._run_allowed_failure(["gh", "release", "delete", tag, "-y"])
        self.rerun(normalized)
        self._run_checked(["gh", "release", "create", tag, "--title", tag, "--generate-notes"])
        return normalized

    def _preflight(self) -> None:
        if not self.project_dir.exists():
            raise VersioningError(f"Directory '{self.project_dir}' does not exist")
        _check_required_tools(["poetry", "git", "gh"])

    def _read_version(self) -> str:
        result = self.runner.run(["poetry", "version", "--short"])
        version = result.stdout.strip()
        if not version:
            raise VersioningError("Unable to read version from poetry")
        return version

    def _release(self, version: str) -> None:
        tag = _tag_name(version)
        self._run_checked(["git", "add", "pyproject.toml"])
        self._run_checked(["git", "add", "-f", "poetry.lock"])
        self._run_checked(["git", "commit", "-m", f"chore(release): {tag}"])
        self._run_checked(["git", "push"])
        self._run_checked(["git", "tag", tag])
        self._run_checked(["git", "push", "origin", tag])
        self._run_checked(["gh", "release", "create", tag, "--title", tag, "--generate-notes"])

    def _run_checked(self, args: list[str]) -> None:
        self.runner.run(args, check=True)

    def _run_allowed_failure(self, args: list[str]) -> None:
        self.runner.run(args, check=False)



"""MCP resource handlers — read-only data exposed to AI assistants."""

from __future__ import annotations

import json
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from fastmcp import FastMCP


def register_resources(mcp: FastMCP) -> None:
    """Register all resource handlers on the given MCP server."""

    @mcp.resource(
        "preset://list",
        name="Available Presets",
        description="List of all available project presets with names and descriptions.",
        mime_type="application/json",
    )
    def preset_list() -> str:
        from pypreset.preset_loader import list_available_presets

        presets = list_available_presets()
        return json.dumps([{"name": name, "description": desc} for name, desc in presets])

    @mcp.resource(
        "config://user",
        name="User Configuration",
        description="Current user-level default configuration values.",
        mime_type="application/json",
    )
    def user_config() -> str:
        from pypreset.user_config import get_config_path, load_user_config

        config = load_user_config()
        return json.dumps(
            {
                "config_path": str(get_config_path()),
                "values": config,
            }
        )

    @mcp.resource(
        "template://list",
        name="Available Templates",
        description="List of Jinja2 template files available for project generation.",
        mime_type="application/json",
    )
    def template_list() -> str:
        from pypreset.template_engine import get_templates_dir

        templates_dir = get_templates_dir()
        templates = (
            sorted(p.name for p in templates_dir.glob("*.j2")) if templates_dir.exists() else []
        )
        return json.dumps(templates)



"""MCP server for pypreset — exposes project scaffolding via Model Context Protocol."""

from fastmcp import FastMCP


def create_server() -> FastMCP:
    """Create and configure the FastMCP server instance."""
    mcp = FastMCP(
        name="pypreset",
        instructions=(
            "MCP server for pypreset, a meta-tool that scaffolds Poetry-based Python projects "
            "from YAML presets. Use tools to create/augment projects, list presets, validate "
            "project structure, and manage user configuration."
        ),
    )

    # Import and register tools, resources, prompts
    from pypreset.mcp_server.prompts import register_prompts
    from pypreset.mcp_server.resources import register_resources
    from pypreset.mcp_server.tools import register_tools

    register_tools(mcp)
    register_resources(mcp)
    register_prompts(mcp)

    return mcp


def main() -> None:
    """Entry point for the pypreset-mcp CLI command."""
    server = create_server()
    server.run()



"""Project generator - creates projects from configurations."""

import logging
import stat
import subprocess
from pathlib import Path

from pypreset.models import (
    CreationPackageManager,
    FileTemplate,
    LayoutStyle,
    ProjectConfig,
)
from pypreset.template_engine import (
    create_jinja_environment,
    get_template_context,
    render_content,
    render_path,
    render_template,
)

logger = logging.getLogger(__name__)


class ProjectGenerator:
    """Generates a project from a configuration."""

    def __init__(self, config: ProjectConfig, output_dir: Path) -> None:
        self.config = config
        self.output_dir = output_dir
        self.project_dir = output_dir / config.metadata.name
        self.env = create_jinja_environment()
        self.context = get_template_context(config)
        self._is_src = config.layout == LayoutStyle.SRC
        self._is_uv = config.package_manager == CreationPackageManager.UV

    @property
    def _package_dir(self) -> Path:
        """Return the package directory based on layout style."""
        package_name = self.context["project"]["package_name"]
        if self._is_src:
            return self.project_dir / "src" / package_name
        return self.project_dir / package_name

    def generate(self) -> Path:
        """Generate the complete project structure."""
        logger.info(f"Generating project '{self.config.metadata.name}' at {self.project_dir}")

        # Create project directory
        self.project_dir.mkdir(parents=True, exist_ok=True)

        # Create directory structure
        self._create_directories()

        # Create files from templates
        self._create_files()

        # Create pyproject.toml
        self._create_pyproject_toml()

        # Create README.md
        self._create_readme()

        # Create additional standard files
        self._create_gitignore()

        # Create GitHub workflows
        self._create_github_workflows()

        # Create pre-commit config if enabled
        if self.config.formatting.pre_commit:
            self._create_pre_commit_config()

        # Create Docker files if enabled
        if self.config.docker.enabled:
            self._create_docker_files()

        # Create devcontainer if enabled
        if self.config.docker.devcontainer:
            self._create_devcontainer()

        logger.info(f"Project '{self.config.metadata.name}' generated successfully")
        return self.project_dir

    def _create_directories(self) -> None:
        """Create all directories in the project structure."""
        # Create the package directory (src-layout: src/pkg, flat-layout: pkg/)
        self._package_dir.mkdir(parents=True, exist_ok=True)

        # Create directories from structure config
        for dir_path in self.config.structure.directories:
            rendered_path = render_path(dir_path, self.context)
            full_path = self.project_dir / rendered_path
            full_path.mkdir(parents=True, exist_ok=True)
            logger.debug(f"Created directory: {full_path}")

        # Create tests directory if testing is enabled
        if self.config.testing.enabled:
            tests_dir = self.project_dir / "tests"
            tests_dir.mkdir(parents=True, exist_ok=True)

    def _create_files(self) -> None:
        """Create all files from templates or inline content."""
        for file_def in self.config.structure.files:
            self._create_file(file_def)

        # Always create __init__.py for the package
        init_path = self._package_dir / "__init__.py"
        if not init_path.exists():
            name = self.config.metadata.name
            version = self.config.metadata.version
            init_content = f'"""{name} package."""\n\n__version__ = "{version}"\n'
            init_path.write_text(init_content)

        # Create tests/__init__.py and test file if testing is enabled
        if self.config.testing.enabled:
            tests_init = self.project_dir / "tests" / "__init__.py"
            if not tests_init.exists():
                tests_init.write_text('"""Tests package."""\n')

            test_file = self.project_dir / "tests" / "test_basic.py"
            if not test_file.exists():
                test_content = self._render_test_file()
                test_file.write_text(test_content)

    def _create_file(self, file_def: FileTemplate) -> None:
        """Create a single file from a template or inline content."""
        rendered_path = render_path(file_def.path, self.context)
        full_path = self.project_dir / rendered_path

        # Ensure parent directory exists
        full_path.parent.mkdir(parents=True, exist_ok=True)

        # Get content from template or inline
        if file_def.template:
            content = render_template(self.env, file_def.template, self.context)
        elif file_def.content:
            content = render_content(file_def.content, self.context)
        else:
            content = ""

        full_path.write_text(content)
        logger.debug(f"Created file: {full_path}")

        # Make executable if needed
        if file_def.executable:
            current_mode = full_path.stat().st_mode
            full_path.chmod(current_mode | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH)

    def _create_pyproject_toml(self) -> None:
        """Create the pyproject.toml file."""
        template = "pyproject_uv.toml.j2" if self._is_uv else "pyproject.toml.j2"
        content = render_template(self.env, template, self.context)
        pyproject_path = self.project_dir / "pyproject.toml"
        pyproject_path.write_text(content)
        logger.debug(f"Created pyproject.toml: {pyproject_path}")

    def _create_readme(self) -> None:
        """Create the README.md file."""
        content = render_template(self.env, "README.md.j2", self.context)
        readme_path = self.project_dir / "README.md"
        readme_path.write_text(content)
        logger.debug(f"Created README.md: {readme_path}")

    def _create_gitignore(self) -> None:
        """Create the .gitignore file."""
        content = render_template(self.env, "gitignore.j2", self.context)
        gitignore_path = self.project_dir / ".gitignore"
        gitignore_path.write_text(content)
        logger.debug(f"Created .gitignore: {gitignore_path}")

    def _create_github_workflows(self) -> None:
        """Create GitHub Actions workflow files."""
        # Only create workflows if testing or formatting is enabled
        if not self.config.testing.enabled and not self.config.formatting.enabled:
            logger.debug("Skipping GitHub workflows - no testing or formatting enabled")
            return

        workflows_dir = self.project_dir / ".github" / "workflows"
        workflows_dir.mkdir(parents=True, exist_ok=True)

        ci_template = "github_ci_uv.yaml.j2" if self._is_uv else "github_ci.yaml.j2"
        content = render_template(self.env, ci_template, self.context)
        ci_path = workflows_dir / "ci.yaml"
        ci_path.write_text(content)
        logger.debug(f"Created GitHub CI workflow: {ci_path}")

        # Create dependabot.yml if enabled
        if self.config.dependabot.enabled:
            self._create_dependabot()

    def _create_dependabot(self) -> None:
        """Create the dependabot.yml configuration file."""
        github_dir = self.project_dir / ".github"
        github_dir.mkdir(parents=True, exist_ok=True)

        content = render_template(self.env, "dependabot.yml.j2", self.context)
        dependabot_path = github_dir / "dependabot.yml"
        dependabot_path.write_text(content)
        logger.debug(f"Created dependabot.yml: {dependabot_path}")

    def _create_pre_commit_config(self) -> None:
        """Create .pre-commit-config.yaml for git hooks."""
        content = render_template(self.env, "pre-commit-config.yaml.j2", self.context)
        config_path = self.project_dir / ".pre-commit-config.yaml"
        config_path.write_text(content)
        logger.debug(f"Created .pre-commit-config.yaml: {config_path}")

    def _create_docker_files(self) -> None:
        """Create Dockerfile and .dockerignore."""
        template = "Dockerfile_uv.j2" if self._is_uv else "Dockerfile.j2"
        content = render_template(self.env, template, self.context)
        dockerfile_path = self.project_dir / "Dockerfile"
        dockerfile_path.write_text(content)
        logger.debug(f"Created Dockerfile: {dockerfile_path}")

        ignore_content = render_template(self.env, "dockerignore.j2", self.context)
        ignore_path = self.project_dir / ".dockerignore"
        ignore_path.write_text(ignore_content)
        logger.debug(f"Created .dockerignore: {ignore_path}")

    def _create_devcontainer(self) -> None:
        """Create .devcontainer/devcontainer.json configuration."""
        devcontainer_dir = self.project_dir / ".devcontainer"
        devcontainer_dir.mkdir(parents=True, exist_ok=True)

        content = render_template(self.env, "devcontainer.json.j2", self.context)
        devcontainer_path = devcontainer_dir / "devcontainer.json"
        devcontainer_path.write_text(content)
        logger.debug(f"Created devcontainer.json: {devcontainer_path}")

    def _render_test_file(self) -> str:
        """Render a basic test file."""
        package_name = self.context["project"]["package_name"]
        return f'''"""Basic tests for {self.config.metadata.name}."""

import {package_name}


def test_version() -> None:
    """Test that version is defined."""
    assert hasattr({package_name}, "__version__")
    assert isinstance({package_name}.__version__, str)


def test_import() -> None:
    """Test that package can be imported."""
    import {package_name}

    assert {package_name} is not None
'''


def generate_project(
    config: ProjectConfig,
    output_dir: Path,
    initialize_git: bool = True,
    install_dependencies: bool = False,
) -> Path:
    """Generate a project from a configuration.

    Args:
        config: The project configuration
        output_dir: Directory to create the project in
        initialize_git: Whether to initialize a git repository
        install_dependencies: Whether to run poetry install

    Returns:
        Path to the generated project directory
    """
    generator = ProjectGenerator(config, output_dir)
    project_dir = generator.generate()

    if initialize_git:
        _init_git(project_dir)

    if install_dependencies:
        is_uv = config.package_manager == CreationPackageManager.UV
        _install_dependencies(project_dir, use_uv=is_uv)

    return project_dir


def _init_git(project_dir: Path) -> None:
    """Initialize a git repository in the project directory."""
    try:
        subprocess.run(
            ["git", "init"],
            cwd=project_dir,
            capture_output=True,
            check=True,
        )
        logger.info("Initialized git repository")
    except subprocess.CalledProcessError as e:
        logger.warning(f"Failed to initialize git repository: {e}")
    except FileNotFoundError:
        logger.warning("git not found, skipping repository initialization")


def _install_dependencies(project_dir: Path, *, use_uv: bool = False) -> None:
    """Run dependency installation in the project directory."""
    cmd = ["uv", "sync"] if use_uv else ["poetry", "install"]
    tool_name = "uv" if use_uv else "poetry"
    try:
        subprocess.run(
            cmd,
            cwd=project_dir,
            capture_output=True,
            check=True,
        )
        logger.info(f"Installed dependencies with {tool_name}")
    except subprocess.CalledProcessError as e:
        logger.warning(f"Failed to install dependencies: {e}")
    except FileNotFoundError:
        logger.warning(f"{tool_name} not found, skipping dependency installation")



"""Preset loading and merging functionality."""

import logging
from pathlib import Path
from typing import Any

import yaml

from pypreset.models import (
    CreationPackageManager,
    DependabotConfig,
    Dependencies,
    DirectoryStructure,
    DockerConfig,
    EntryPoint,
    FileTemplate,
    FormattingConfig,
    LayoutStyle,
    Metadata,
    OverrideOptions,
    PresetConfig,
    ProjectConfig,
    TestingConfig,
    TypingLevel,
)
from pypreset.user_config import apply_user_defaults

logger = logging.getLogger(__name__)


def get_builtin_presets_dir() -> Path:
    """Get the directory containing built-in presets."""
    return Path(__file__).parent / "presets"


def get_user_presets_dir() -> Path:
    """Get the user's custom presets directory."""
    return Path.home() / ".config" / "pypreset" / "presets"


def load_yaml_file(path: Path) -> dict[str, Any]:
    """Load a YAML file and return its contents."""
    with open(path) as f:
        return yaml.safe_load(f) or {}


def deep_merge(base: dict[str, Any], override: dict[str, Any]) -> dict[str, Any]:
    """Deep merge two dictionaries, with override taking precedence.

    None values in override are skipped (treated as "not set").
    """
    result = base.copy()
    for key, value in override.items():
        # Skip None values (they mean "not set" in partial configs)
        if value is None:
            continue
        if key in result and isinstance(result[key], dict) and isinstance(value, dict):
            result[key] = deep_merge(result[key], value)  # type: ignore[arg-type]
        elif key in result and isinstance(result[key], list) and isinstance(value, list):
            # For lists, extend rather than replace
            result[key] = result[key] + value
        else:
            result[key] = value
    return result


def find_preset_file(preset_name: str, custom_path: Path | None = None) -> Path | None:
    """Find a preset file by name."""
    if custom_path and custom_path.exists():
        return custom_path

    # Check user presets first
    user_preset = get_user_presets_dir() / f"{preset_name}.yaml"
    if user_preset.exists():
        return user_preset

    # Check built-in presets
    builtin_preset = get_builtin_presets_dir() / f"{preset_name}.yaml"
    if builtin_preset.exists():
        return builtin_preset

    return None


def list_available_presets() -> list[tuple[str, str]]:
    """List all available presets with their descriptions."""
    presets: list[tuple[str, str]] = []
    seen_names: set[str] = set()

    # Check user presets first (they take precedence)
    user_dir = get_user_presets_dir()
    if user_dir.exists():
        for preset_file in user_dir.glob("*.yaml"):
            name = preset_file.stem
            if name not in seen_names:
                data = load_yaml_file(preset_file)
                description = data.get("description", "")
                presets.append((name, f"{description} (user)"))
                seen_names.add(name)

    # Check built-in presets
    builtin_dir = get_builtin_presets_dir()
    if builtin_dir.exists():
        for preset_file in builtin_dir.glob("*.yaml"):
            name = preset_file.stem
            if name not in seen_names:
                data = load_yaml_file(preset_file)
                description = data.get("description", "")
                presets.append((name, description))
                seen_names.add(name)

    return sorted(presets, key=lambda x: x[0])


def load_preset(preset_name: str, custom_path: Path | None = None) -> PresetConfig:
    """Load a preset configuration."""
    preset_path = find_preset_file(preset_name, custom_path)
    if preset_path is None:
        raise ValueError(f"Preset '{preset_name}' not found")

    data = load_yaml_file(preset_path)
    return PresetConfig(**data)


def resolve_preset_chain(preset: PresetConfig) -> dict[str, Any]:
    """Resolve a preset's inheritance chain and return merged config."""
    if preset.base is None:
        return preset.model_dump(exclude={"name", "description", "base"})

    # Load and resolve base preset
    base_preset = load_preset(preset.base)
    base_config = resolve_preset_chain(base_preset)

    # Merge current preset on top
    current_config = preset.model_dump(exclude={"name", "description", "base"})
    return deep_merge(base_config, current_config)


def _set_nested(config: dict[str, Any], section: str, key: str, value: Any) -> None:
    """Set a value in a nested config section, creating the section if needed."""
    if section not in config:
        config[section] = {}
    config[section][key] = value


def _extend_dep_list(config: dict[str, Any], group: str, packages: list[str]) -> None:
    """Extend a dependency list, creating the structure if needed."""
    deps = config.setdefault("dependencies", {})
    deps.setdefault(group, []).extend(packages)


def apply_overrides(config: dict[str, Any], overrides: OverrideOptions) -> dict[str, Any]:
    """Apply runtime overrides to a configuration."""
    result = config.copy()

    # Nested overrides: (value, section, key) — applied when value is not None
    _nested_overrides: list[tuple[Any, str, str]] = [
        (overrides.testing_enabled, "testing", "enabled"),
        (overrides.formatting_enabled, "formatting", "enabled"),
        (overrides.radon_enabled, "formatting", "radon"),
        (overrides.pre_commit_enabled, "formatting", "pre_commit"),
        (overrides.version_bumping_enabled, "formatting", "version_bumping"),
        (overrides.python_version, "metadata", "python_version"),
        (overrides.docker_enabled, "docker", "enabled"),
        (overrides.devcontainer_enabled, "docker", "devcontainer"),
    ]
    for value, section, key in _nested_overrides:
        if value is not None:
            _set_nested(result, section, key, value)

    # Enum overrides stored via .value
    if overrides.type_checker is not None:
        _set_nested(result, "formatting", "type_checker", overrides.type_checker.value)

    # Top-level enum overrides
    if overrides.typing_level is not None:
        result["typing_level"] = overrides.typing_level.value
    if overrides.layout is not None:
        result["layout"] = overrides.layout.value
    if overrides.package_manager is not None:
        result["package_manager"] = overrides.package_manager.value

    # Dependency list extensions
    if overrides.extra_packages:
        _extend_dep_list(result, "main", overrides.extra_packages)
    if overrides.extra_dev_packages:
        _extend_dep_list(result, "dev", overrides.extra_dev_packages)

    return result


def build_project_config(
    project_name: str,
    preset_name: str,
    overrides: OverrideOptions | None = None,
    custom_preset_path: Path | None = None,
) -> ProjectConfig:
    """Build a complete project configuration from a preset and overrides."""
    # Load and resolve the preset
    preset = load_preset(preset_name, custom_preset_path)
    config = resolve_preset_chain(preset)

    # Apply user-level defaults (lowest priority — presets override these)
    config = apply_user_defaults(config)

    # Apply runtime overrides (highest priority)
    if overrides:
        config = apply_overrides(config, overrides)

    # Set the project name in metadata
    if "metadata" not in config:
        config["metadata"] = {}
    config["metadata"]["name"] = project_name

    # Replace placeholders in entry points
    package_name = project_name.replace("-", "_")
    config = _replace_placeholders(config, project_name, package_name)

    # Convert to ProjectConfig
    return _dict_to_project_config(config)


def _replace_placeholders(
    config: dict[str, Any], project_name: str, package_name: str
) -> dict[str, Any]:
    """Replace __PROJECT_NAME__ and __PACKAGE_NAME__ placeholders in config."""
    result = config.copy()

    # Replace in entry points
    if "entry_points" in result:
        new_entry_points = []
        for ep in result["entry_points"]:
            new_ep = {}
            for key, value in ep.items():
                if isinstance(value, str):
                    value = value.replace("__PROJECT_NAME__", project_name)
                    value = value.replace("__PACKAGE_NAME__", package_name)
                new_ep[key] = value
            new_entry_points.append(new_ep)
        result["entry_points"] = new_entry_points

    return result


def _strip_none_values(data: dict[str, Any]) -> dict[str, Any]:
    """Recursively strip None values from a dictionary."""
    result = {}
    for key, value in data.items():
        if value is None:
            continue
        if isinstance(value, dict):
            result[key] = _strip_none_values(value)
        else:
            result[key] = value
    return result


def _dict_to_project_config(data: dict[str, Any]) -> ProjectConfig:
    """Convert a dictionary to a ProjectConfig object."""
    # Strip None values from the entire config
    data = _strip_none_values(data)

    # Build metadata
    metadata_data = data.get("metadata", {})
    metadata = Metadata(**metadata_data)

    # Build structure
    structure_data = data.get("structure", {})
    files = [FileTemplate(**f) for f in structure_data.get("files", [])]
    structure = DirectoryStructure(
        directories=structure_data.get("directories", []),
        files=files,
    )

    # Build dependencies
    deps_data = data.get("dependencies", {})
    dependencies = Dependencies(
        main=deps_data.get("main", []),
        dev=deps_data.get("dev", []),
        optional=deps_data.get("optional", {}),
    )

    # Build testing config
    testing_data = data.get("testing", {})
    testing = TestingConfig(**testing_data) if testing_data else TestingConfig()  # type: ignore[call-arg]

    # Build formatting config
    formatting_data = data.get("formatting", {})
    formatting = FormattingConfig(**formatting_data) if formatting_data else FormattingConfig()  # type: ignore[call-arg]

    # Build dependabot config
    dependabot_data = data.get("dependabot", {})
    dependabot = DependabotConfig(**dependabot_data) if dependabot_data else DependabotConfig()  # type: ignore[call-arg]

    # Build docker config
    docker_data = data.get("docker", {})
    docker = DockerConfig(**docker_data) if docker_data else DockerConfig()  # type: ignore[call-arg]

    # Build entry points
    entry_points = [EntryPoint(**ep) for ep in data.get("entry_points", [])]

    # Build typing level
    typing_level_str = data.get("typing_level", "strict")
    typing_level = TypingLevel(typing_level_str) if typing_level_str else TypingLevel.STRICT

    # Build layout style
    layout_str = data.get("layout", "src")
    layout = LayoutStyle(layout_str) if layout_str else LayoutStyle.SRC

    # Build package manager
    pm_str = data.get("package_manager", "poetry")
    package_manager = CreationPackageManager(pm_str) if pm_str else CreationPackageManager.POETRY

    return ProjectConfig(
        metadata=metadata,
        structure=structure,
        dependencies=dependencies,
        testing=testing,
        formatting=formatting,
        dependabot=dependabot,
        docker=docker,
        typing_level=typing_level,
        layout=layout,
        package_manager=package_manager,
        entry_points=entry_points,
        extras=data.get("extras", {}),
    )



"""Project validation utilities."""

import logging
import subprocess
from pathlib import Path

logger = logging.getLogger(__name__)


class ValidationResult:
    """Result of a validation check."""

    def __init__(self, passed: bool, message: str, details: str | None = None) -> None:
        self.passed = passed
        self.message = message
        self.details = details

    def __bool__(self) -> bool:
        return self.passed

    def __repr__(self) -> str:
        status = "✓" if self.passed else "✗"
        return f"{status} {self.message}"


class ProjectValidator:
    """Validates that a generated project is structurally sound."""

    def __init__(self, project_dir: Path) -> None:
        self.project_dir = project_dir
        self.results: list[ValidationResult] = []

    def validate_all(self) -> list[ValidationResult]:
        """Run all validation checks."""
        self.results = []

        self.results.append(self._check_project_exists())
        self.results.append(self._check_pyproject_toml())
        self.results.append(self._check_src_layout())
        self.results.append(self._check_package_init())
        self.results.append(self._check_readme())
        self.results.append(self._check_gitignore())
        self.results.append(self._check_pyproject_toml_valid())

        return self.results

    def is_valid(self) -> bool:
        """Check if all validations passed."""
        if not self.results:
            self.validate_all()
        return all(self.results)

    def _check_project_exists(self) -> ValidationResult:
        """Check that the project directory exists."""
        if self.project_dir.exists() and self.project_dir.is_dir():
            return ValidationResult(True, "Project directory exists")
        return ValidationResult(False, "Project directory does not exist")

    def _check_pyproject_toml(self) -> ValidationResult:
        """Check that pyproject.toml exists."""
        pyproject_path = self.project_dir / "pyproject.toml"
        if pyproject_path.exists():
            return ValidationResult(True, "pyproject.toml exists")
        return ValidationResult(False, "pyproject.toml is missing")

    def _check_src_layout(self) -> ValidationResult:
        """Check that a valid package layout (src or flat) is present."""
        src_dir = self.project_dir / "src"
        if src_dir.exists() and src_dir.is_dir():
            packages = [d for d in src_dir.iterdir() if d.is_dir() and not d.name.startswith(".")]
            if packages:
                pkg_names = [p.name for p in packages]
                return ValidationResult(True, f"src-layout with package(s): {pkg_names}")
            return ValidationResult(False, "src directory exists but no packages found")

        # Check for flat layout: top-level directories with __init__.py
        _non_package_dirs = {
            "tests",
            "docs",
            "scripts",
            "data",
            "notebooks",
            "models",
            "reports",
        }
        if self.project_dir.exists():
            flat_packages = [
                d
                for d in self.project_dir.iterdir()
                if d.is_dir()
                and not d.name.startswith((".", "_"))
                and d.name not in _non_package_dirs
                and (d / "__init__.py").exists()
            ]
            if flat_packages:
                pkg_names = [p.name for p in flat_packages]
                return ValidationResult(True, f"flat-layout with package(s): {pkg_names}")

        return ValidationResult(
            False,
            "No package directory found (neither src-layout nor flat-layout)",
        )

    def _check_package_init(self) -> ValidationResult:
        """Check that package __init__.py exists."""
        src_dir = self.project_dir / "src"

        # Try src-layout first
        if src_dir.exists():
            packages = [d for d in src_dir.iterdir() if d.is_dir() and not d.name.startswith(".")]
            if packages:
                for package in packages:
                    init_file = package / "__init__.py"
                    if not init_file.exists():
                        return ValidationResult(False, f"__init__.py missing in src/{package.name}")
                return ValidationResult(True, "All packages have __init__.py")

        # Try flat-layout
        _non_package_dirs = {
            "tests",
            "docs",
            "scripts",
            "data",
            "notebooks",
            "models",
            "reports",
        }
        if self.project_dir.exists():
            flat_packages = [
                d
                for d in self.project_dir.iterdir()
                if d.is_dir()
                and not d.name.startswith((".", "_"))
                and d.name not in _non_package_dirs
                and (d / "__init__.py").exists()
            ]
            if flat_packages:
                return ValidationResult(True, "All packages have __init__.py")

        return ValidationResult(False, "No packages with __init__.py found")

    def _check_readme(self) -> ValidationResult:
        """Check that README.md exists."""
        readme_path = self.project_dir / "README.md"
        if readme_path.exists():
            return ValidationResult(True, "README.md exists")
        return ValidationResult(False, "README.md is missing")

    def _check_gitignore(self) -> ValidationResult:
        """Check that .gitignore exists."""
        gitignore_path = self.project_dir / ".gitignore"
        if gitignore_path.exists():
            return ValidationResult(True, ".gitignore exists")
        return ValidationResult(False, ".gitignore is missing")

    def _check_pyproject_toml_valid(self) -> ValidationResult:
        """Check that pyproject.toml is valid TOML."""
        pyproject_path = self.project_dir / "pyproject.toml"
        if not pyproject_path.exists():
            return ValidationResult(False, "Cannot validate: pyproject.toml missing")

        try:
            import tomllib

            with open(pyproject_path, "rb") as f:
                tomllib.load(f)
            return ValidationResult(True, "pyproject.toml is valid TOML")
        except Exception as e:
            return ValidationResult(False, f"pyproject.toml is invalid: {e}")


def validate_project(project_dir: Path) -> tuple[bool, list[ValidationResult]]:
    """Validate a project directory.

    Returns:
        Tuple of (is_valid, list of validation results)
    """
    validator = ProjectValidator(project_dir)
    results = validator.validate_all()
    return validator.is_valid(), results


def validate_with_poetry(project_dir: Path) -> ValidationResult:
    """Validate project using poetry check."""
    try:
        result = subprocess.run(
            ["poetry", "check"],
            cwd=project_dir,
            capture_output=True,
            text=True,
        )
        if result.returncode == 0:
            return ValidationResult(True, "poetry check passed", result.stdout)
        return ValidationResult(False, "poetry check failed", result.stderr)
    except FileNotFoundError:
        return ValidationResult(False, "poetry not found", "Cannot run poetry check")
    except Exception as e:
        return ValidationResult(False, f"poetry check error: {e}")



"""Augment generator - adds components to existing projects."""

import logging
from abc import ABC, abstractmethod
from dataclasses import dataclass
from enum import StrEnum
from pathlib import Path
from typing import Any

from jinja2 import Environment, FileSystemLoader

from pypreset.docker_utils import resolve_docker_base_image as _resolve_base_image
from pypreset.interactive_prompts import AugmentConfig
from pypreset.project_analyzer import (
    DetectedLinter,
    DetectedTestFramework,
    DetectedTypeChecker,
)

logger = logging.getLogger(__name__)


class AugmentComponent(StrEnum):
    """Available augment components."""

    TEST_WORKFLOW = "test_workflow"
    LINT_WORKFLOW = "lint_workflow"
    DEPENDABOT = "dependabot"
    TESTS_DIR = "tests_dir"
    GITIGNORE = "gitignore"
    CONFTEST = "conftest"
    PRE_COMMIT = "pre_commit"
    PYPI_PUBLISH = "pypi_publish"
    DOCKERFILE = "dockerfile"
    DEVCONTAINER = "devcontainer"


@dataclass
class GeneratedFile:
    """Represents a generated file."""

    path: Path
    content: str
    overwritten: bool = False


@dataclass
class AugmentResult:
    """Result of an augment operation."""

    success: bool
    files_created: list[GeneratedFile]
    files_skipped: list[Path]
    errors: list[str]


def get_augment_templates_dir() -> Path:
    """Get the directory containing augment templates."""
    return Path(__file__).parent / "templates" / "augment"


def create_augment_jinja_env() -> Environment:
    """Create Jinja2 environment for augment templates."""
    # Try augment-specific templates first, fall back to main templates
    augment_dir = get_augment_templates_dir()
    templates_dir = Path(__file__).parent / "templates"

    # Create the augment directory if it doesn't exist
    augment_dir.mkdir(parents=True, exist_ok=True)

    return Environment(
        loader=FileSystemLoader([str(augment_dir), str(templates_dir)]),
        trim_blocks=True,
        lstrip_blocks=True,
        keep_trailing_newline=True,
    )


def get_augment_context(config: AugmentConfig) -> dict[str, Any]:
    """Build template context from augment config."""
    return {
        "project": {
            "name": config.project_name,
            "package_name": config.package_name,
            "python_version": config.python_version,
            "description": config.description,
        },
        "testing": {
            "enabled": config.test_framework != DetectedTestFramework.NONE,
            "framework": config.test_framework.value,
            "coverage": config.has_coverage,
        },
        "formatting": {
            "enabled": config.linter != DetectedLinter.NONE,
            "tool": config.linter.value,
            "line_length": config.line_length,
        },
        "typing": {
            "enabled": config.type_checker != DetectedTypeChecker.NONE,
            "checker": config.type_checker.value,
        },
        "dependabot": {
            "enabled": config.generate_dependabot,
            "schedule": config.dependabot_schedule,
            "open_pull_requests_limit": config.dependabot_pr_limit,
        },
        "source_dirs": config.source_dirs,
        "has_src_layout": config.has_src_layout,
        "layout": "src" if config.has_src_layout else "flat",
        "package_manager": config.package_manager.value,
        "docker": {
            "base_image": _resolve_base_image(config.python_version),
        },
    }


class ComponentGenerator(ABC):
    """Base class for component generators."""

    def __init__(self, project_dir: Path, config: AugmentConfig) -> None:
        self.project_dir = project_dir
        self.config = config
        self.env = create_augment_jinja_env()
        self.context = get_augment_context(config)

    @property
    @abstractmethod
    def component_name(self) -> str:
        """Return the component name for logging."""
        ...

    @abstractmethod
    def should_generate(self) -> bool:
        """Check if this component should be generated."""
        ...

    @abstractmethod
    def generate(self, force: bool = False) -> list[GeneratedFile]:
        """Generate the component files."""
        ...

    def _render_template(self, template_name: str) -> str:
        """Render a template with the current context."""
        template = self.env.get_template(template_name)
        return template.render(**self.context)

    def _write_file(self, path: Path, content: str, force: bool = False) -> GeneratedFile | None:
        """Write content to a file, optionally overwriting."""
        full_path = self.project_dir / path

        # Ensure parent directory exists
        full_path.parent.mkdir(parents=True, exist_ok=True)

        overwritten = full_path.exists()
        if overwritten and not force:
            logger.info(f"Skipping {path} (already exists)")
            return None

        full_path.write_text(content)
        logger.info(f"{'Overwrote' if overwritten else 'Created'} {path}")

        return GeneratedFile(path=path, content=content, overwritten=overwritten)


class TestWorkflowGenerator(ComponentGenerator):
    """Generates test workflow for GitHub Actions."""

    @property
    def component_name(self) -> str:
        return "test_workflow"

    def should_generate(self) -> bool:
        return self.config.generate_test_workflow

    def generate(self, force: bool = False) -> list[GeneratedFile]:
        files: list[GeneratedFile] = []

        if self.config.test_framework == DetectedTestFramework.NONE:
            logger.warning("Skipping test workflow - no test framework configured")
            return files

        content = self._render_template("test_workflow.yaml.j2")
        path = Path(".github/workflows/test.yaml")

        result = self._write_file(path, content, force)
        if result:
            files.append(result)

        return files


class LintWorkflowGenerator(ComponentGenerator):
    """Generates lint workflow for GitHub Actions."""

    @property
    def component_name(self) -> str:
        return "lint_workflow"

    def should_generate(self) -> bool:
        return self.config.generate_lint_workflow

    def generate(self, force: bool = False) -> list[GeneratedFile]:
        files: list[GeneratedFile] = []

        if (
            self.config.linter == DetectedLinter.NONE
            and self.config.type_checker == DetectedTypeChecker.NONE
        ):
            logger.warning("Skipping lint workflow - no linter or type checker configured")
            return files

        content = self._render_template("lint_workflow.yaml.j2")
        path = Path(".github/workflows/lint.yaml")

        result = self._write_file(path, content, force)
        if result:
            files.append(result)

        return files


class DependabotGenerator(ComponentGenerator):
    """Generates dependabot.yml configuration."""

    @property
    def component_name(self) -> str:
        return "dependabot"

    def should_generate(self) -> bool:
        return self.config.generate_dependabot

    def generate(self, force: bool = False) -> list[GeneratedFile]:
        files: list[GeneratedFile] = []

        content = self._render_template("dependabot_augment.yml.j2")
        path = Path(".github/dependabot.yml")

        result = self._write_file(path, content, force)
        if result:
            files.append(result)

        return files


class TestsDirectoryGenerator(ComponentGenerator):
    """Generates tests directory and template test file."""

    @property
    def component_name(self) -> str:
        return "tests_dir"

    def should_generate(self) -> bool:
        return self.config.generate_tests_dir

    def generate(self, force: bool = False) -> list[GeneratedFile]:
        files: list[GeneratedFile] = []

        # Create tests/__init__.py
        init_path = Path("tests/__init__.py")
        init_content = '"""Tests package."""\n'
        result = self._write_file(
            init_path, init_content, force=False
        )  # Never overwrite __init__.py
        if result:
            files.append(result)

        # Create tests/conftest.py
        conftest_content = self._render_template("conftest.py.j2")
        conftest_path = Path("tests/conftest.py")
        result = self._write_file(conftest_path, conftest_content, force=False)
        if result:
            files.append(result)

        # Create tests/test_basic.py
        test_content = self._render_template("test_template.py.j2")
        test_path = Path("tests/test_basic.py")
        result = self._write_file(test_path, test_content, force=False)
        if result:
            files.append(result)

        return files


class GitignoreGenerator(ComponentGenerator):
    """Generates .gitignore file."""

    @property
    def component_name(self) -> str:
        return "gitignore"

    def should_generate(self) -> bool:
        return self.config.generate_gitignore

    def generate(self, force: bool = False) -> list[GeneratedFile]:
        if not self.should_generate():
            return []

        files: list[GeneratedFile] = []

        content = self._render_template("gitignore.j2")
        path = Path(".gitignore")

        result = self._write_file(path, content, force)
        if result:
            files.append(result)

        return files


class PypiPublishWorkflowGenerator(ComponentGenerator):
    """Generates PyPI publish workflow for GitHub Actions."""

    @property
    def component_name(self) -> str:
        return "pypi_publish"

    def should_generate(self) -> bool:
        return self.config.generate_pypi_publish

    def generate(self, force: bool = False) -> list[GeneratedFile]:
        files: list[GeneratedFile] = []

        content = self._render_template("pypi_publish_workflow.yaml.j2")
        path = Path(".github/workflows/publish.yaml")

        result = self._write_file(path, content, force)
        if result:
            files.append(result)

        return files


class DockerfileGenerator(ComponentGenerator):
    """Generates Dockerfile and .dockerignore."""

    @property
    def component_name(self) -> str:
        return "dockerfile"

    def should_generate(self) -> bool:
        return self.config.generate_dockerfile

    def generate(self, force: bool = False) -> list[GeneratedFile]:
        files: list[GeneratedFile] = []

        # Select template based on package manager
        if self.config.package_manager.value == "uv":
            template_name = "Dockerfile_uv.j2"
        else:
            template_name = "Dockerfile.j2"

        content = self._render_template(template_name)
        result = self._write_file(Path("Dockerfile"), content, force)
        if result:
            files.append(result)

        ignore_content = self._render_template("dockerignore.j2")
        result = self._write_file(Path(".dockerignore"), ignore_content, force)
        if result:
            files.append(result)

        return files


class DevcontainerGenerator(ComponentGenerator):
    """Generates .devcontainer/devcontainer.json."""

    @property
    def component_name(self) -> str:
        return "devcontainer"

    def should_generate(self) -> bool:
        return self.config.generate_devcontainer

    def generate(self, force: bool = False) -> list[GeneratedFile]:
        files: list[GeneratedFile] = []

        content = self._render_template("devcontainer.json.j2")
        result = self._write_file(Path(".devcontainer/devcontainer.json"), content, force)
        if result:
            files.append(result)

        return files


class AugmentOrchestrator:
    """Orchestrates the augment operation across components."""

    def __init__(self, project_dir: Path, config: AugmentConfig) -> None:
        self.project_dir = project_dir.absolute()
        self.config = config

        # Register all component generators
        self.generators: list[ComponentGenerator] = [
            TestWorkflowGenerator(self.project_dir, config),
            LintWorkflowGenerator(self.project_dir, config),
            DependabotGenerator(self.project_dir, config),
            TestsDirectoryGenerator(self.project_dir, config),
            GitignoreGenerator(self.project_dir, config),
            PypiPublishWorkflowGenerator(self.project_dir, config),
            DockerfileGenerator(self.project_dir, config),
            DevcontainerGenerator(self.project_dir, config),
        ]

    def run(
        self, force: bool = False, components: list[AugmentComponent] | None = None
    ) -> AugmentResult:
        """Run the augment operation."""
        files_created: list[GeneratedFile] = []
        files_skipped: list[Path] = []
        errors: list[str] = []

        for generator in self.generators:
            # Skip if specific components requested and this isn't one of them
            if components is not None:
                component_enum = AugmentComponent(generator.component_name)
                if component_enum not in components:
                    continue

            if not generator.should_generate():
                logger.debug(f"Skipping {generator.component_name} - not enabled")
                continue

            try:
                generated = generator.generate(force=force)
                files_created.extend(generated)
            except Exception as e:
                logger.error(f"Error generating {generator.component_name}: {e}")
                errors.append(f"{generator.component_name}: {str(e)}")

        return AugmentResult(
            success=len(errors) == 0,
            files_created=files_created,
            files_skipped=files_skipped,
            errors=errors,
        )


def augment_project(
    project_dir: Path,
    config: AugmentConfig,
    force: bool = False,
    components: list[AugmentComponent] | None = None,
) -> AugmentResult:
    """Augment an existing project with workflows and tests.

    Args:
        project_dir: Path to the project directory
        config: Augment configuration
        force: Whether to overwrite existing files
        components: Specific components to generate (None = all enabled)

    Returns:
        AugmentResult with details of generated files
    """
    orchestrator = AugmentOrchestrator(project_dir, config)
    return orchestrator.run(force=force, components=components)



"""MCP prompt handlers — guided workflows for AI assistants."""

from __future__ import annotations

import json
from typing import TYPE_CHECKING, Annotated

from fastmcp.prompts import Message
from pydantic import Field

if TYPE_CHECKING:
    from fastmcp import FastMCP


def register_prompts(mcp: FastMCP) -> None:
    """Register all prompt handlers on the given MCP server."""

    @mcp.prompt(
        name="create-project",
        description="Guided workflow for creating a new Python project from a preset.",
        tags={"project", "create"},
    )
    def create_project_prompt(
        project_name: Annotated[
            str | None, Field(description="Project name, if already known")
        ] = None,
        preset: Annotated[str | None, Field(description="Preset name, if already chosen")] = None,
    ) -> list[Message]:
        from pypreset.preset_loader import list_available_presets

        presets = list_available_presets()
        preset_info = json.dumps(
            [{"name": name, "description": desc} for name, desc in presets],
            indent=2,
        )

        instructions = [
            "You are helping the user create a new Python project using pypreset.",
            "",
            f"Available presets:\n{preset_info}",
            "",
            "Guide the user through these steps:",
            "1. Choose a project name (valid Python package name with hyphens)",
            "2. Select a preset from the list above",
            "3. Ask about optional overrides:",
            "   - Layout style (src or flat)",
            "   - Package manager (poetry or uv)",
            "   - Type checker (mypy, ty, or none)",
            "   - Typing level (none, basic, or strict)",
            "   - Python version",
            "4. Ask for the output directory",
            "5. Call the create_project tool with the collected parameters",
            "6. Optionally validate the generated project with validate_project",
        ]

        if project_name:
            instructions.append(f"\nThe user has already chosen the name: {project_name}")
        if preset:
            instructions.append(f"The user has already chosen the preset: {preset}")

        return [
            Message(role="user", content="\n".join(instructions)),
            Message(
                role="assistant",
                content=(
                    "I'll help you create a new Python project. Let me guide you through the setup."
                ),
            ),
        ]

    @mcp.prompt(
        name="augment-project",
        description=(
            "Guided workflow for augmenting an existing Python project with CI, tests, etc."
        ),
        tags={"project", "augment"},
    )
    def augment_project_prompt(
        project_dir: Annotated[
            str | None, Field(description="Path to the project directory, if known")
        ] = None,
    ) -> list[Message]:
        instructions = [
            "You are helping the user augment an existing Python project using pypreset.",
            "",
            "Guide the user through these steps:",
            "1. Confirm the project directory path",
            "2. Explain which components can be added:",
            "   - Test CI workflow (GitHub Actions)",
            "   - Lint CI workflow (GitHub Actions)",
            "   - Dependabot configuration",
            "   - Tests directory with template test files",
            "   - .gitignore file",
            "3. Ask which components they want to generate",
            "4. Ask if existing files should be overwritten (force mode)",
            "5. Call the augment_project tool with the selected options",
        ]

        if project_dir:
            instructions.append(f"\nThe user's project is at: {project_dir}")

        return [
            Message(role="user", content="\n".join(instructions)),
            Message(
                role="assistant",
                content=(
                    "I'll help you add CI, tests, and other components to your existing project. "
                    "Let me guide you through the options."
                ),
            ),
        ]



"""Interactive prompts for filling in missing configuration."""

import logging
from dataclasses import dataclass
from typing import Any

from rich import print as rprint
from rich.console import Console
from rich.prompt import Confirm, Prompt
from rich.table import Table

from pypreset.project_analyzer import (
    DetectedLinter,
    DetectedTestFramework,
    DetectedTypeChecker,
    DetectedValue,
    PackageManager,
    ProjectAnalysis,
)

logger = logging.getLogger(__name__)
console = Console()


@dataclass
class AugmentConfig:
    """Configuration for the augment operation."""

    # Project metadata
    project_name: str
    package_name: str
    python_version: str
    description: str

    # Package manager
    package_manager: PackageManager

    # Testing configuration
    test_framework: DetectedTestFramework
    has_coverage: bool

    # Linting configuration
    linter: DetectedLinter
    type_checker: DetectedTypeChecker
    line_length: int

    # Source directories
    source_dirs: list[str]
    has_src_layout: bool

    # What to generate
    generate_test_workflow: bool = True
    generate_lint_workflow: bool = True
    generate_dependabot: bool = True
    generate_tests_dir: bool = True
    generate_gitignore: bool = True
    generate_pypi_publish: bool = False
    generate_dockerfile: bool = False
    generate_devcontainer: bool = False

    # Dependabot settings
    dependabot_schedule: str = "weekly"
    dependabot_pr_limit: int = 5


class InteractivePrompter:
    """Interactive session for collecting missing configuration."""

    def __init__(self, analysis: ProjectAnalysis) -> None:
        self.analysis = analysis
        self.confirmed_values: dict[str, Any] = {}

    def display_analysis_summary(self) -> None:
        """Display a summary of what was detected."""
        rprint("\n[bold cyan]📊 Project Analysis Summary[/bold cyan]\n")

        table = Table(show_header=True, header_style="bold magenta")
        table.add_column("Setting", style="cyan")
        table.add_column("Detected Value")
        table.add_column("Confidence")
        table.add_column("Source")

        detections: list[tuple[str, DetectedValue[Any] | None]] = [
            ("Project Name", self.analysis.project_name),
            ("Package Name", self.analysis.package_name),
            ("Version", self.analysis.version),
            ("Python Version", self.analysis.python_version),
            ("Package Manager", self.analysis.package_manager),
            ("Test Framework", self.analysis.test_framework),
            ("Linter", self.analysis.linter),
            ("Type Checker", self.analysis.type_checker),
            ("Line Length", self.analysis.line_length),
        ]

        for name, detection in detections:
            if detection is not None:
                value = detection.value
                if hasattr(value, "value"):  # Enum
                    value = value.value

                confidence_style = {
                    "high": "[green]high[/green]",
                    "medium": "[yellow]medium[/yellow]",
                    "low": "[red]low[/red]",
                }.get(detection.confidence, detection.confidence)

                table.add_row(name, str(value), confidence_style, detection.source)
            else:
                table.add_row(name, "[dim]Not detected[/dim]", "[red]none[/red]", "-")

        console.print(table)

        # Show existing structure
        rprint("\n[bold cyan]📁 Project Structure[/bold cyan]")
        rprint(f"  • Has src/ layout: {'✓' if self.analysis.has_src_layout else '✗'}")
        rprint(f"  • Has tests/ directory: {'✓' if self.analysis.has_tests_dir else '✗'}")
        rprint(f"  • Has .github/ directory: {'✓' if self.analysis.has_github_dir else '✗'}")
        rprint(f"  • Has dependabot.yml: {'✓' if self.analysis.has_dependabot else '✗'}")
        rprint(f"  • Has .gitignore: {'✓' if self.analysis.has_gitignore else '✗'}")

        if self.analysis.existing_workflows:
            rprint(f"  • Existing workflows: {[w.name for w in self.analysis.existing_workflows]}")

        if self.analysis.existing_tests:
            rprint(f"  • Existing tests: {len(self.analysis.existing_tests)} files")

    def prompt_for_missing_fields(self) -> dict[str, Any]:
        """Prompt user for missing fields."""
        if not self.analysis.missing_fields:
            return {}

        rprint(
            "\n[bold yellow]⚠️ Some configuration could not be detected."
            " Please provide:[/bold yellow]\n"
        )

        results: dict[str, Any] = {}

        for field in self.analysis.missing_fields:
            if field.choices:
                # Use choices prompt
                choices_str = ", ".join(field.choices)
                default_display = f" [{field.default}]" if field.default else ""
                value = Prompt.ask(
                    f"  {field.description} ({choices_str}){default_display}",
                    default=field.default,
                    choices=field.choices,
                )
            else:
                # Free text prompt
                default_display = f" [{field.default}]" if field.default else ""
                value = Prompt.ask(
                    f"  {field.description}{default_display}",
                    default=field.default or "",
                )

            results[field.name] = value

        return results

    def prompt_for_confirmation(self) -> dict[str, Any]:
        """Prompt user to confirm uncertain detected values."""
        uncertain = self.analysis.get_uncertain_values()

        if not uncertain:
            return {}

        rprint("\n[bold yellow]🔍 Please confirm these detected values:[/bold yellow]\n")

        results: dict[str, Any] = {}

        for name, detection in uncertain.items():
            value = detection.value
            display_value = value.value if hasattr(value, "value") else str(value)

            confirmed = Confirm.ask(
                f"  {name}: [cyan]{display_value}[/cyan] (from {detection.source})?",
                default=True,
            )

            if confirmed:
                results[name] = detection.value
            else:
                # Allow user to provide alternative
                new_value = Prompt.ask(f"  Enter new value for {name}")
                results[name] = new_value

        return results

    def prompt_for_components(self) -> dict[str, bool]:
        """Prompt user for which components to generate."""
        rprint("\n[bold cyan]🛠️ Select components to generate:[/bold cyan]\n")

        components: dict[str, bool] = {}

        # Test workflow
        if self.analysis.existing_workflows:
            has_test_workflow = any(
                "test" in w.name.lower() or "ci" in w.name.lower()
                for w in self.analysis.existing_workflows
            )
            if has_test_workflow:
                rprint("  [dim]Test workflow already exists[/dim]")
                components["generate_test_workflow"] = Confirm.ask(
                    "  Overwrite existing test workflow?",
                    default=False,
                )
            else:
                components["generate_test_workflow"] = Confirm.ask(
                    "  Generate test workflow?",
                    default=True,
                )
        else:
            components["generate_test_workflow"] = Confirm.ask(
                "  Generate test workflow?",
                default=True,
            )

        # Lint workflow
        has_lint_workflow = any("lint" in w.name.lower() for w in self.analysis.existing_workflows)
        if has_lint_workflow:
            rprint("  [dim]Lint workflow already exists[/dim]")
            components["generate_lint_workflow"] = Confirm.ask(
                "  Overwrite existing lint workflow?",
                default=False,
            )
        else:
            components["generate_lint_workflow"] = Confirm.ask(
                "  Generate lint workflow?",
                default=True,
            )

        # Dependabot
        if self.analysis.has_dependabot:
            rprint("  [dim]dependabot.yml already exists[/dim]")
            components["generate_dependabot"] = Confirm.ask(
                "  Overwrite existing dependabot.yml?",
                default=False,
            )
        else:
            components["generate_dependabot"] = Confirm.ask(
                "  Generate dependabot.yml?",
                default=True,
            )

        # Tests directory
        if self.analysis.has_tests_dir and self.analysis.existing_tests:
            test_count = len(self.analysis.existing_tests)
            rprint(f"  [dim]tests/ directory exists with {test_count} test files[/dim]")
            components["generate_tests_dir"] = Confirm.ask(
                "  Generate template test file? (will not overwrite existing)",
                default=False,
            )
        elif self.analysis.has_tests_dir:
            components["generate_tests_dir"] = Confirm.ask(
                "  Generate template test file in tests/?",
                default=True,
            )
        else:
            components["generate_tests_dir"] = Confirm.ask(
                "  Create tests/ directory with template test?",
                default=True,
            )

        # Gitignore
        if self.analysis.has_gitignore:
            rprint("  [dim].gitignore already exists[/dim]")
            components["generate_gitignore"] = Confirm.ask(
                "  Overwrite existing .gitignore?",
                default=False,
            )
        else:
            components["generate_gitignore"] = Confirm.ask(
                "  Generate .gitignore?",
                default=True,
            )

        return components

    def prompt_for_dependabot_config(self) -> dict[str, Any]:
        """Prompt for dependabot configuration if enabled."""
        rprint("\n[bold cyan]📦 Dependabot Configuration:[/bold cyan]\n")

        schedule = Prompt.ask(
            "  Update schedule (daily, weekly, monthly)",
            default="weekly",
            choices=["daily", "weekly", "monthly"],
        )

        pr_limit = Prompt.ask(
            "  Maximum open pull requests",
            default="5",
        )

        return {
            "dependabot_schedule": schedule,
            "dependabot_pr_limit": int(pr_limit),
        }

    def build_augment_config(self, interactive: bool = True) -> AugmentConfig:
        """Build augment configuration through interactive prompts or auto-detection."""
        # Display analysis summary
        if interactive:
            self.display_analysis_summary()

        # Collect missing fields
        if interactive and self.analysis.missing_fields:
            missing_values = self.prompt_for_missing_fields()
        else:
            missing_values = {}

        # Confirm uncertain values
        confirmed_values = self.prompt_for_confirmation() if interactive else {}

        # Select components to generate
        if interactive:
            components = self.prompt_for_components()
        else:
            components = {
                "generate_test_workflow": True,
                "generate_lint_workflow": True,
                "generate_dependabot": not self.analysis.has_dependabot,
                "generate_tests_dir": not self.analysis.has_tests_dir,
                "generate_gitignore": not self.analysis.has_gitignore,
                "generate_pypi_publish": False,
            }

        # Dependabot configuration
        if components.get("generate_dependabot", False) and interactive:
            dependabot_config = self.prompt_for_dependabot_config()
        else:
            dependabot_config = {"dependabot_schedule": "weekly", "dependabot_pr_limit": 5}

        # Build the final config
        return self._build_config(missing_values, confirmed_values, components, dependabot_config)

    def _build_config(
        self,
        missing_values: dict[str, Any],
        confirmed_values: dict[str, Any],
        components: dict[str, bool],
        dependabot_config: dict[str, Any],
    ) -> AugmentConfig:
        """Build AugmentConfig from collected values."""

        # Helper to get value with priority: confirmed > missing > detected
        def get_value(name: str, default: Any = None) -> Any:
            if name in confirmed_values:
                return confirmed_values[name]
            if name in missing_values:
                return missing_values[name]

            detected = getattr(self.analysis, name, None)
            if detected is not None and isinstance(detected, DetectedValue):
                return detected.value
            return default

        # Get test framework
        test_framework_value = get_value("test_framework", DetectedTestFramework.PYTEST)
        if isinstance(test_framework_value, str):
            test_framework = DetectedTestFramework(test_framework_value)
        else:
            test_framework = test_framework_value

        # Get linter
        linter_value = get_value("linter", DetectedLinter.RUFF)
        linter = DetectedLinter(linter_value) if isinstance(linter_value, str) else linter_value

        # Get type checker
        type_checker_value = get_value("type_checker", DetectedTypeChecker.MYPY)
        if isinstance(type_checker_value, str):
            type_checker = DetectedTypeChecker(type_checker_value)
        else:
            type_checker = type_checker_value

        # Get package manager
        package_manager_value = get_value("package_manager", PackageManager.POETRY)
        if isinstance(package_manager_value, str):
            package_manager = PackageManager(package_manager_value)
        else:
            package_manager = package_manager_value

        return AugmentConfig(
            project_name=get_value("project_name", "my-project"),
            package_name=get_value("package_name", "my_project"),
            python_version=get_value("python_version", "3.11"),
            description=get_value("description", ""),
            package_manager=package_manager,
            test_framework=test_framework,
            has_coverage="pytest-cov" in self.analysis.dev_dependencies
            or "coverage" in self.analysis.dev_dependencies,
            linter=linter,
            type_checker=type_checker,
            line_length=get_value("line_length", 100),
            source_dirs=self.analysis.source_dirs or ["src"],
            has_src_layout=self.analysis.has_src_layout,
            generate_test_workflow=components.get("generate_test_workflow", True),
            generate_lint_workflow=components.get("generate_lint_workflow", True),
            generate_dependabot=components.get("generate_dependabot", True),
            generate_tests_dir=components.get("generate_tests_dir", True),
            generate_gitignore=components.get("generate_gitignore", True),
            generate_pypi_publish=components.get("generate_pypi_publish", False),
            generate_dockerfile=components.get("generate_dockerfile", False),
            generate_devcontainer=components.get("generate_devcontainer", False),
            dependabot_schedule=dependabot_config.get("dependabot_schedule", "weekly"),
            dependabot_pr_limit=dependabot_config.get("dependabot_pr_limit", 5),
        )


def run_interactive_session(analysis: ProjectAnalysis) -> AugmentConfig:
    """Run an interactive session to collect configuration."""
    prompter = InteractivePrompter(analysis)
    return prompter.build_augment_config(interactive=True)


def run_auto_session(analysis: ProjectAnalysis) -> AugmentConfig:
    """Build configuration automatically without prompts."""
    prompter = InteractivePrompter(analysis)
    return prompter.build_augment_config(interactive=False)



"""Docker utilities for resolving base images."""


def resolve_docker_base_image(python_version: str, base_image: str | None = None) -> str:
    """Resolve the Docker base image for a project.

    Returns base_image if provided, otherwise derives from python_version.
    """
    if base_image:
        return base_image
    return f"python:{python_version}-slim"



"""MCP tool handlers — actions an AI assistant can invoke."""

from __future__ import annotations

import json
from pathlib import Path
from typing import TYPE_CHECKING, Annotated, Any

from pydantic import Field

if TYPE_CHECKING:
    from fastmcp import FastMCP

from pypreset.models import (
    CreationPackageManager,
    LayoutStyle,
    OverrideOptions,
    TypeChecker,
    TypingLevel,
)


def register_tools(mcp: FastMCP) -> None:
    """Register all tool handlers on the given MCP server."""

    # ------------------------------------------------------------------
    # create_project
    # ------------------------------------------------------------------
    @mcp.tool(
        name="create_project",
        description=(
            "Create a new Python project from a preset. "
            "Returns the path to the generated project directory."
        ),
        tags={"project", "create"},
    )
    def create_project(
        project_name: Annotated[str, Field(description="Name for the new project (e.g. 'my-app')")],
        preset: Annotated[
            str, Field(description="Preset name (e.g. 'empty-package', 'cli-tool', 'data-science')")
        ],
        output_dir: Annotated[str, Field(description="Directory to create the project in")] = ".",
        initialize_git: Annotated[bool, Field(description="Initialize a git repository")] = True,
        install_dependencies: Annotated[
            bool, Field(description="Run package manager install after generation")
        ] = False,
        layout: Annotated[
            str | None, Field(description="Layout style override: 'src' or 'flat'")
        ] = None,
        type_checker: Annotated[
            str | None,
            Field(description="Type checker override: 'mypy', 'pyright', 'ty', or 'none'"),
        ] = None,
        package_manager: Annotated[
            str | None, Field(description="Package manager override: 'poetry' or 'uv'")
        ] = None,
        typing_level: Annotated[
            str | None,
            Field(description="Typing strictness override: 'none', 'basic', or 'strict'"),
        ] = None,
        python_version: Annotated[
            str | None, Field(description="Python version override (e.g. '3.12')")
        ] = None,
        docker: Annotated[bool, Field(description="Generate Dockerfile and .dockerignore")] = False,
        devcontainer: Annotated[
            bool, Field(description="Generate .devcontainer/ configuration")
        ] = False,
    ) -> str:
        from pypreset.generator import generate_project
        from pypreset.preset_loader import build_project_config

        overrides = OverrideOptions(
            testing_enabled=None,
            formatting_enabled=None,
            radon_enabled=None,
            pre_commit_enabled=None,
            version_bumping_enabled=None,
            layout=LayoutStyle(layout) if layout else None,
            type_checker=TypeChecker(type_checker) if type_checker else None,
            package_manager=CreationPackageManager(package_manager) if package_manager else None,
            typing_level=TypingLevel(typing_level) if typing_level else None,
            python_version=python_version,
            docker_enabled=docker if docker else None,
            devcontainer_enabled=devcontainer if devcontainer else None,
        )

        config = build_project_config(
            project_name=project_name,
            preset_name=preset,
            overrides=overrides,
        )

        project_path = generate_project(
            config=config,
            output_dir=Path(output_dir),
            initialize_git=initialize_git,
            install_dependencies=install_dependencies,
        )

        return json.dumps(
            {
                "project_dir": str(project_path),
                "project_name": project_name,
                "preset": preset,
                "layout": config.layout.value,
                "package_manager": config.package_manager.value,
                "docker_enabled": config.docker.enabled,
                "devcontainer_enabled": config.docker.devcontainer,
            }
        )

    # ------------------------------------------------------------------
    # validate_project
    # ------------------------------------------------------------------
    @mcp.tool(
        name="validate_project",
        description="Validate the structural correctness of a generated project directory.",
        tags={"project", "validate"},
    )
    def validate_project(
        project_dir: Annotated[str, Field(description="Path to the project directory to validate")],
    ) -> str:
        from pypreset.validator import validate_project as _validate

        is_valid, results = _validate(Path(project_dir))

        return json.dumps(
            {
                "valid": is_valid,
                "checks": [
                    {
                        "passed": r.passed,
                        "message": r.message,
                        "details": r.details,
                    }
                    for r in results
                ],
            }
        )

    # ------------------------------------------------------------------
    # list_presets
    # ------------------------------------------------------------------
    @mcp.tool(
        name="list_presets",
        description="List all available project presets with their descriptions.",
        tags={"preset"},
    )
    def list_presets() -> str:
        from pypreset.preset_loader import list_available_presets

        presets = list_available_presets()
        return json.dumps([{"name": name, "description": desc} for name, desc in presets])

    # ------------------------------------------------------------------
    # show_preset
    # ------------------------------------------------------------------
    @mcp.tool(
        name="show_preset",
        description="Show the full configuration of a specific preset.",
        tags={"preset"},
    )
    def show_preset(
        preset_name: Annotated[str, Field(description="Name of the preset to show")],
    ) -> str:
        from pypreset.preset_loader import load_preset

        preset = load_preset(preset_name)
        return json.dumps(preset.model_dump(exclude_none=True), default=str)

    # ------------------------------------------------------------------
    # get_user_config
    # ------------------------------------------------------------------
    @mcp.tool(
        name="get_user_config",
        description="Read the current user-level default configuration.",
        tags={"config"},
    )
    def get_user_config() -> str:
        from pypreset.user_config import get_config_path, load_user_config

        config = load_user_config()
        return json.dumps(
            {
                "config_path": str(get_config_path()),
                "values": config,
            }
        )

    # ------------------------------------------------------------------
    # set_user_config
    # ------------------------------------------------------------------
    @mcp.tool(
        name="set_user_config",
        description=(
            "Update user-level default configuration values. "
            "Merges provided keys into existing config."
        ),
        tags={"config"},
    )
    def set_user_config(
        values: Annotated[
            dict[str, Any],
            Field(
                description=(
                    "Key-value pairs to set (e.g. {'layout': 'src', 'python_version': '3.12'})"
                )
            ),
        ],
    ) -> str:
        from pypreset.user_config import load_user_config, save_user_config

        current = load_user_config()
        current.update(values)
        path = save_user_config(current)
        return json.dumps({"config_path": str(path), "values": current})

    # ------------------------------------------------------------------
    # augment_project
    # ------------------------------------------------------------------
    @mcp.tool(
        name="augment_project",
        description=(
            "Add CI workflows, tests, gitignore, and/or dependabot to an existing project. "
            "Analyzes the project first, then generates selected components."
        ),
        tags={"project", "augment"},
    )
    def augment_project(
        project_dir: Annotated[str, Field(description="Path to the existing project directory")],
        force: Annotated[bool, Field(description="Overwrite existing files if they exist")] = False,
        generate_test_workflow: Annotated[
            bool, Field(description="Generate test CI workflow")
        ] = True,
        generate_lint_workflow: Annotated[
            bool, Field(description="Generate lint CI workflow")
        ] = True,
        generate_dependabot: Annotated[bool, Field(description="Generate dependabot.yml")] = True,
        generate_tests_dir: Annotated[bool, Field(description="Generate tests directory")] = True,
        generate_gitignore: Annotated[bool, Field(description="Generate .gitignore")] = True,
        generate_dockerfile: Annotated[
            bool, Field(description="Generate Dockerfile and .dockerignore")
        ] = False,
        generate_devcontainer: Annotated[
            bool, Field(description="Generate .devcontainer/ configuration")
        ] = False,
    ) -> str:
        from pypreset.augment_generator import augment_project as _augment
        from pypreset.interactive_prompts import InteractivePrompter
        from pypreset.project_analyzer import analyze_project

        path = Path(project_dir)
        analysis = analyze_project(path)

        # Build config non-interactively
        prompter = InteractivePrompter(analysis)
        config = prompter.build_augment_config(interactive=False)

        # Override component selections
        config.generate_test_workflow = generate_test_workflow
        config.generate_lint_workflow = generate_lint_workflow
        config.generate_dependabot = generate_dependabot
        config.generate_tests_dir = generate_tests_dir
        config.generate_gitignore = generate_gitignore
        config.generate_dockerfile = generate_dockerfile
        config.generate_devcontainer = generate_devcontainer

        result = _augment(project_dir=path, config=config, force=force)

        return json.dumps(
            {
                "success": result.success,
                "files_created": [
                    {"path": str(f.path), "overwritten": f.overwritten}
                    for f in result.files_created
                ],
                "files_skipped": [str(p) for p in result.files_skipped],
                "errors": result.errors,
            }
        )



"""Configuration models for pypreset."""

from enum import StrEnum
from typing import Any, Literal

from pydantic import BaseModel, Field


class LayoutStyle(StrEnum):
    """Project directory layout style.

    See https://packaging.python.org/en/latest/discussions/src-layout-vs-flat-layout/
    """

    SRC = "src"
    FLAT = "flat"


class TypingLevel(StrEnum):
    """Python typing strictness level."""

    NONE = "none"
    BASIC = "basic"
    STRICT = "strict"


class TestingFramework(StrEnum):
    """Supported testing frameworks."""

    PYTEST = "pytest"
    UNITTEST = "unittest"
    NONE = "none"


class FormattingTool(StrEnum):
    """Supported formatting/linting tools."""

    RUFF = "ruff"
    BLACK = "black"
    NONE = "none"


class TypeChecker(StrEnum):
    """Supported type checking tools."""

    MYPY = "mypy"
    PYRIGHT = "pyright"
    TY = "ty"
    NONE = "none"


class CreationPackageManager(StrEnum):
    """Package manager for project creation."""

    POETRY = "poetry"
    UV = "uv"


class FileTemplate(BaseModel):
    """A file template definition."""

    path: str = Field(..., description="Relative path within the project")
    template: str | None = Field(None, description="Jinja2 template name to use")
    content: str | None = Field(None, description="Inline content (if no template)")
    executable: bool = Field(False, description="Whether the file should be executable")


class DirectoryStructure(BaseModel):
    """Directory structure definition."""

    directories: list[str] = Field(
        default_factory=list, description="List of directories to create"
    )
    files: list[FileTemplate] = Field(default_factory=list, description="List of files to create")  # type: ignore[arg-type]


class DependencyGroup(BaseModel):
    """A group of dependencies."""

    packages: list[str] = Field(default_factory=list, description="List of packages")


class Dependencies(BaseModel):
    """Project dependencies configuration."""

    main: list[str] = Field(default_factory=list, description="Main dependencies")
    dev: list[str] = Field(default_factory=list, description="Development dependencies")
    optional: dict[str, list[str]] = Field(
        default_factory=dict, description="Optional dependency groups"
    )


class TestingConfig(BaseModel):
    """Testing configuration."""

    enabled: bool = Field(True, description="Whether testing is enabled")
    framework: TestingFramework = Field(
        TestingFramework.PYTEST, description="Testing framework to use"
    )
    coverage: bool = Field(False, description="Whether to include coverage tools")


class FormattingConfig(BaseModel):
    """Code formatting configuration."""

    enabled: bool = Field(True, description="Whether formatting is enabled")
    tool: FormattingTool = Field(FormattingTool.RUFF, description="Formatting tool to use")
    line_length: int = Field(100, description="Maximum line length")
    radon: bool = Field(False, description="Enable radon complexity checking")
    pre_commit: bool = Field(False, description="Generate pre-commit hooks config")
    version_bumping: bool = Field(
        False, description="Include bump-my-version for version management"
    )
    type_checker: TypeChecker = Field(TypeChecker.MYPY, description="Type checking tool to use")


class DependabotConfig(BaseModel):
    """Dependabot configuration for automatic dependency updates."""

    enabled: bool = Field(True, description="Whether to generate dependabot.yml")
    schedule: Literal["daily", "weekly", "monthly"] = Field(
        "weekly", description="Update check frequency"
    )
    open_pull_requests_limit: int = Field(
        5, description="Maximum number of open PRs for version updates"
    )


class DockerConfig(BaseModel):
    """Docker configuration for generating Dockerfile and .dockerignore."""

    enabled: bool = Field(False, description="Whether to generate Dockerfile and .dockerignore")
    base_image: str | None = Field(
        None, description="Base image override (auto-resolved from python_version if None)"
    )
    devcontainer: bool = Field(False, description="Whether to generate .devcontainer/ config")


class Metadata(BaseModel):
    """Project metadata (mirrors pyproject.toml)."""

    name: str = Field(..., description="Project name")
    version: str = Field("0.1.0", description="Project version")
    description: str = Field("", description="Project description")
    authors: list[str] = Field(default_factory=list, description="Project authors")
    license: str | None = Field(None, description="Project license")
    readme: str = Field("README.md", description="Readme file")
    python_version: str = Field("3.11", description="Minimum Python version")
    keywords: list[str] = Field(default_factory=list, description="Project keywords")
    classifiers: list[str] = Field(default_factory=list, description="PyPI classifiers")


class EntryPoint(BaseModel):
    """Script entry point configuration."""

    name: str = Field(..., description="Command name")
    module: str = Field(..., description="Module path (e.g., 'mypackage.cli:app')")


# Partial models for presets (all fields optional for merging)
class PartialMetadata(BaseModel):
    """Partial metadata for preset configs (all fields optional)."""

    name: str | None = Field(None, description="Project name")
    version: str | None = Field(None, description="Project version")
    description: str | None = Field(None, description="Project description")
    authors: list[str] | None = Field(None, description="Project authors")
    license: str | None = Field(None, description="Project license")
    readme: str | None = Field(None, description="Readme file")
    python_version: str | None = Field(None, description="Minimum Python version")
    keywords: list[str] | None = Field(None, description="Project keywords")
    classifiers: list[str] | None = Field(None, description="PyPI classifiers")


class PartialDirectoryStructure(BaseModel):
    """Partial directory structure for preset configs."""

    directories: list[str] | None = Field(None, description="List of directories to create")
    files: list[FileTemplate] | None = Field(None, description="List of files to create")


class PartialDependencies(BaseModel):
    """Partial dependencies for preset configs."""

    main: list[str] | None = Field(None, description="Main dependencies")
    dev: list[str] | None = Field(None, description="Development dependencies")
    optional: dict[str, list[str]] | None = Field(None, description="Optional dependency groups")


class PartialTestingConfig(BaseModel):
    """Partial testing config for preset configs."""

    enabled: bool | None = Field(None, description="Whether testing is enabled")
    framework: TestingFramework | None = Field(None, description="Testing framework to use")
    coverage: bool | None = Field(None, description="Whether to include coverage tools")


class PartialFormattingConfig(BaseModel):
    """Partial formatting config for preset configs."""

    enabled: bool | None = Field(None, description="Whether formatting is enabled")
    tool: FormattingTool | None = Field(None, description="Formatting tool to use")
    line_length: int | None = Field(None, description="Maximum line length")
    radon: bool | None = Field(None, description="Enable radon complexity checking")
    pre_commit: bool | None = Field(None, description="Generate pre-commit hooks config")
    version_bumping: bool | None = Field(
        None, description="Include bump-my-version for version management"
    )
    type_checker: TypeChecker | None = Field(None, description="Type checking tool to use")


class PartialDependabotConfig(BaseModel):
    """Partial dependabot config for preset configs."""

    enabled: bool | None = Field(None, description="Whether to generate dependabot.yml")
    schedule: Literal["daily", "weekly", "monthly"] | None = Field(
        None, description="Update check frequency"
    )
    open_pull_requests_limit: int | None = Field(
        None, description="Maximum number of open PRs for version updates"
    )


class PartialDockerConfig(BaseModel):
    """Partial docker config for preset configs."""

    enabled: bool | None = Field(
        None, description="Whether to generate Dockerfile and .dockerignore"
    )
    base_image: str | None = Field(None, description="Base image override")
    devcontainer: bool | None = Field(None, description="Whether to generate .devcontainer/ config")


class ProjectConfig(BaseModel):
    """Complete project configuration."""

    metadata: Metadata
    structure: DirectoryStructure = Field(default_factory=DirectoryStructure)  # type: ignore[arg-type]
    dependencies: Dependencies = Field(default_factory=Dependencies)  # type: ignore[arg-type]
    testing: TestingConfig = Field(default_factory=TestingConfig)  # type: ignore[arg-type]
    formatting: FormattingConfig = Field(default_factory=FormattingConfig)  # type: ignore[arg-type]
    dependabot: DependabotConfig = Field(default_factory=DependabotConfig)  # type: ignore[arg-type]
    docker: DockerConfig = Field(default_factory=DockerConfig)  # type: ignore[arg-type]
    typing_level: TypingLevel = Field(TypingLevel.STRICT, description="Typing strictness")
    layout: LayoutStyle = Field(LayoutStyle.SRC, description="Project layout style (src or flat)")
    package_manager: CreationPackageManager = Field(
        CreationPackageManager.POETRY, description="Package manager"
    )
    entry_points: list[EntryPoint] = Field(default_factory=list, description="Script entry points")  # type: ignore[arg-type]
    extras: dict[str, Any] = Field(default_factory=dict, description="Additional configuration")


class PresetConfig(BaseModel):
    """Preset configuration that can be extended/overridden."""

    name: str = Field(..., description="Preset name")
    description: str = Field("", description="Preset description")
    base: str | None = Field(None, description="Base preset to extend")

    # Typed partial configs for better schema generation
    metadata: PartialMetadata = Field(default_factory=PartialMetadata)  # type: ignore[arg-type]
    structure: PartialDirectoryStructure = Field(default_factory=PartialDirectoryStructure)  # type: ignore[arg-type]
    dependencies: PartialDependencies = Field(default_factory=PartialDependencies)  # type: ignore[arg-type]
    testing: PartialTestingConfig = Field(default_factory=PartialTestingConfig)  # type: ignore[arg-type]
    formatting: PartialFormattingConfig = Field(default_factory=PartialFormattingConfig)  # type: ignore[arg-type]
    dependabot: PartialDependabotConfig = Field(default_factory=PartialDependabotConfig)  # type: ignore[arg-type]
    docker: PartialDockerConfig = Field(default_factory=PartialDockerConfig)  # type: ignore[arg-type]
    typing_level: TypingLevel | None = None
    layout: LayoutStyle | None = None
    package_manager: CreationPackageManager | None = None
    entry_points: list[EntryPoint] = Field(default_factory=list, description="Script entry points")  # type: ignore[arg-type]
    extras: dict[str, Any] = Field(default_factory=dict, description="Additional configuration")


class OverrideOptions(BaseModel):
    """Options that can override preset defaults at runtime."""

    testing_enabled: bool | None = Field(None, description="Override testing enabled")
    formatting_enabled: bool | None = Field(None, description="Override formatting enabled")
    radon_enabled: bool | None = Field(None, description="Override radon complexity checking")
    pre_commit_enabled: bool | None = Field(None, description="Override pre-commit hooks")
    version_bumping_enabled: bool | None = Field(
        None, description="Override bump-my-version inclusion"
    )
    python_version: str | None = Field(None, description="Override Python version")
    layout: LayoutStyle | None = Field(None, description="Override layout style (src or flat)")
    extra_packages: list[str] = Field(default_factory=list, description="Additional packages")
    extra_dev_packages: list[str] = Field(
        default_factory=list, description="Additional dev packages"
    )
    typing_level: TypingLevel | None = Field(None, description="Override typing level")
    type_checker: TypeChecker | None = Field(None, description="Override type checker")
    package_manager: CreationPackageManager | None = Field(
        None, description="Override package manager"
    )
    docker_enabled: bool | None = Field(None, description="Override Docker generation")
    devcontainer_enabled: bool | None = Field(None, description="Override devcontainer generation")



"""User-level configuration for pypreset defaults.

Reads from ~/.config/pypreset/config.yaml and provides defaults
that are applied during project creation and augmentation.
"""

import logging
from enum import Enum
from pathlib import Path
from typing import Any

import yaml

from pypreset.models import (
    CreationPackageManager,
    FormattingTool,
    LayoutStyle,
    TestingFramework,
    TypeChecker,
    TypingLevel,
)

logger = logging.getLogger(__name__)

CONFIG_DIR = Path.home() / ".config" / "pypreset"
CONFIG_FILE = CONFIG_DIR / "config.yaml"

# Keys that map to enum types for validation
_ENUM_FIELDS: dict[str, type[Enum]] = {
    "layout": LayoutStyle,
    "typing_level": TypingLevel,
    "formatter": FormattingTool,
    "testing_framework": TestingFramework,
    "type_checker": TypeChecker,
    "package_manager": CreationPackageManager,
}


def get_config_path() -> Path:
    """Return the path to the user config file."""
    return CONFIG_FILE


def load_user_config() -> dict[str, Any]:
    """Load user configuration from disk.

    Returns an empty dict if the file doesn't exist or is invalid.
    """
    config_path = get_config_path()
    if not config_path.exists():
        return {}

    try:
        with open(config_path) as f:
            data = yaml.safe_load(f) or {}
    except Exception as e:
        logger.warning(f"Failed to load user config from {config_path}: {e}")
        return {}

    if not isinstance(data, dict):
        logger.warning(f"User config is not a mapping: {config_path}")
        return {}

    # Validate enum fields
    validated: dict[str, Any] = {}
    for key, value in data.items():
        if key in _ENUM_FIELDS and value is not None:
            try:
                _ENUM_FIELDS[key](value)
            except ValueError:
                valid = [e.value for e in _ENUM_FIELDS[key]]
                logger.warning(
                    f"Invalid value '{value}' for '{key}' in user config. Valid: {valid}"
                )
                continue
        validated[key] = value

    return validated


def apply_user_defaults(config: dict[str, Any]) -> dict[str, Any]:
    """Apply user defaults to a project config dict.

    User defaults are applied as a base layer — preset values and CLI
    overrides take precedence (they are applied after this).
    """
    user_cfg = load_user_config()
    if not user_cfg:
        return config

    result = config.copy()

    # python_version → metadata.python_version
    if "python_version" in user_cfg:
        metadata = result.setdefault("metadata", {})
        metadata.setdefault("python_version", user_cfg["python_version"])

    # layout
    if "layout" in user_cfg:
        result.setdefault("layout", user_cfg["layout"])

    # typing_level
    if "typing_level" in user_cfg:
        result.setdefault("typing_level", user_cfg["typing_level"])

    # formatter → formatting.tool
    if "formatter" in user_cfg:
        formatting = result.setdefault("formatting", {})
        formatting.setdefault("tool", user_cfg["formatter"])

    # line_length → formatting.line_length
    if "line_length" in user_cfg:
        formatting = result.setdefault("formatting", {})
        formatting.setdefault("line_length", user_cfg["line_length"])

    # testing_framework → testing.framework
    if "testing_framework" in user_cfg:
        testing = result.setdefault("testing", {})
        testing.setdefault("framework", user_cfg["testing_framework"])

    # type_checker → formatting.type_checker
    if "type_checker" in user_cfg:
        formatting = result.setdefault("formatting", {})
        formatting.setdefault("type_checker", user_cfg["type_checker"])

    # package_manager
    if "package_manager" in user_cfg:
        result.setdefault("package_manager", user_cfg["package_manager"])

    return result


def save_user_config(config: dict[str, Any]) -> Path:
    """Save configuration to the user config file.

    Returns the path written to.
    """
    config_path = get_config_path()
    config_path.parent.mkdir(parents=True, exist_ok=True)
    with open(config_path, "w") as f:
        yaml.safe_dump(config, f, default_flow_style=False, sort_keys=False)
    return config_path


def get_default_config_template() -> dict[str, Any]:
    """Return a commented example config for scaffolding."""
    return {
        "python_version": "3.12",
        "layout": "src",
        "typing_level": "strict",
        "formatter": "ruff",
        "line_length": 100,
        "testing_framework": "pytest",
        "type_checker": "mypy",
        "package_manager": "poetry",
    }



"""Project analyzer - extracts metadata from existing Python projects."""

import logging
import re
from dataclasses import dataclass, field
from enum import StrEnum
from pathlib import Path
from typing import Any, Literal

try:
    import tomllib
except ImportError:
    import tomli as tomllib  # type: ignore[import-not-found]


logger = logging.getLogger(__name__)


class PackageManager(StrEnum):
    """Detected package manager."""

    POETRY = "poetry"
    PDM = "pdm"
    SETUPTOOLS = "setuptools"
    HATCH = "hatch"
    FLIT = "flit"
    UNKNOWN = "unknown"


class DetectedLinter(StrEnum):
    """Detected linting/formatting tools."""

    RUFF = "ruff"
    BLACK = "black"
    FLAKE8 = "flake8"
    ISORT = "isort"
    NONE = "none"


class DetectedTestFramework(StrEnum):
    """Detected testing framework."""

    PYTEST = "pytest"
    UNITTEST = "unittest"
    NONE = "none"


class DetectedTypeChecker(StrEnum):
    """Detected type checker."""

    MYPY = "mypy"
    PYRIGHT = "pyright"
    NONE = "none"


@dataclass
class AnalysisConfidence:
    """Confidence levels for detected values."""

    HIGH: Literal["high"] = "high"
    MEDIUM: Literal["medium"] = "medium"
    LOW: Literal["low"] = "low"


@dataclass
class DetectedValue[T]:
    """A detected value with confidence level."""

    value: T
    confidence: Literal["high", "medium", "low"]
    source: str  # Where this was detected from

    @property
    def is_reliable(self) -> bool:
        """Check if value is reliable enough to use without confirmation."""
        return self.confidence == "high"


@dataclass
class MissingField:
    """Represents a field that could not be detected."""

    name: str
    description: str
    required: bool
    default: Any = None
    choices: list[str] | None = None


@dataclass
class ProjectAnalysis:
    """Complete analysis of an existing project."""

    # Project identification
    project_name: DetectedValue[str] | None = None
    package_name: DetectedValue[str] | None = None
    version: DetectedValue[str] | None = None
    description: DetectedValue[str] | None = None
    python_version: DetectedValue[str] | None = None

    # Package management
    package_manager: DetectedValue[PackageManager] | None = None

    # Testing
    test_framework: DetectedValue[DetectedTestFramework] | None = None
    has_tests_dir: bool = False
    existing_tests: list[Path] = field(default_factory=list)

    # Linting/Formatting
    linter: DetectedValue[DetectedLinter] | None = None
    type_checker: DetectedValue[DetectedTypeChecker] | None = None
    line_length: DetectedValue[int] | None = None

    # GitHub workflows
    has_github_dir: bool = False
    existing_workflows: list[Path] = field(default_factory=list)
    has_dependabot: bool = False

    # Gitignore
    has_gitignore: bool = False

    # Structure
    has_src_layout: bool = False
    source_dirs: list[str] = field(default_factory=list)

    # Dependencies (for reference)
    main_dependencies: list[str] = field(default_factory=list)
    dev_dependencies: list[str] = field(default_factory=list)

    # Missing fields that need user input
    missing_fields: list[MissingField] = field(default_factory=list)

    def get_reliable_values(self) -> dict[str, Any]:
        """Get all values that are reliable enough to use without confirmation."""
        result: dict[str, Any] = {}
        for attr_name in [
            "project_name",
            "package_name",
            "version",
            "description",
            "python_version",
            "package_manager",
            "test_framework",
            "linter",
            "type_checker",
            "line_length",
        ]:
            detected = getattr(self, attr_name)
            if detected is not None and detected.is_reliable:
                result[attr_name] = detected.value
        return result

    def get_uncertain_values(self) -> dict[str, DetectedValue[Any]]:
        """Get values that were detected but need confirmation."""
        result: dict[str, DetectedValue[Any]] = {}
        for attr_name in [
            "project_name",
            "package_name",
            "version",
            "description",
            "python_version",
            "package_manager",
            "test_framework",
            "linter",
            "type_checker",
            "line_length",
        ]:
            detected = getattr(self, attr_name)
            if detected is not None and not detected.is_reliable:
                result[attr_name] = detected
        return result


class ProjectAnalyzer:
    """Analyzes an existing Python project to extract metadata and configuration."""

    def __init__(self, project_dir: Path) -> None:
        self.project_dir = project_dir.absolute()
        self.pyproject_data: dict[str, Any] = {}

    def analyze(self) -> ProjectAnalysis:
        """Perform complete project analysis."""
        analysis = ProjectAnalysis()

        # Load pyproject.toml if it exists
        pyproject_path = self.project_dir / "pyproject.toml"
        if pyproject_path.exists():
            self._load_pyproject(pyproject_path)
        else:
            analysis.missing_fields.append(
                MissingField(
                    name="pyproject.toml",
                    description="No pyproject.toml found - project metadata cannot be detected",
                    required=True,
                )
            )
            return analysis

        # Detect package manager
        analysis.package_manager = self._detect_package_manager()

        # Extract project metadata
        analysis.project_name = self._extract_project_name()
        analysis.package_name = self._extract_package_name(analysis.project_name)
        analysis.version = self._extract_version()
        analysis.description = self._extract_description()
        analysis.python_version = self._extract_python_version()

        # Extract dependencies
        analysis.main_dependencies, analysis.dev_dependencies = self._extract_dependencies()

        # Detect testing setup
        analysis.test_framework = self._detect_test_framework(analysis.dev_dependencies)
        analysis.has_tests_dir = (self.project_dir / "tests").is_dir()
        analysis.existing_tests = self._find_existing_tests()

        # Detect linting/formatting
        analysis.linter = self._detect_linter(analysis.dev_dependencies)
        analysis.type_checker = self._detect_type_checker(analysis.dev_dependencies)
        analysis.line_length = self._detect_line_length()

        # Check GitHub structure
        github_dir = self.project_dir / ".github"
        analysis.has_github_dir = github_dir.is_dir()
        if analysis.has_github_dir:
            workflows_dir = github_dir / "workflows"
            if workflows_dir.is_dir():
                analysis.existing_workflows = list(workflows_dir.glob("*.yml")) + list(
                    workflows_dir.glob("*.yaml")
                )
            analysis.has_dependabot = (github_dir / "dependabot.yml").exists()

        # Check gitignore
        analysis.has_gitignore = (self.project_dir / ".gitignore").exists()

        # Check source layout
        analysis.has_src_layout = (self.project_dir / "src").is_dir()
        analysis.source_dirs = self._detect_source_dirs()

        # Determine missing required fields
        analysis.missing_fields = self._determine_missing_fields(analysis)

        return analysis

    def _load_pyproject(self, path: Path) -> None:
        """Load and parse pyproject.toml."""
        try:
            with open(path, "rb") as f:
                self.pyproject_data = tomllib.load(f)
        except Exception as e:
            logger.warning(f"Failed to parse pyproject.toml: {e}")
            self.pyproject_data = {}

    def _detect_package_manager(self) -> DetectedValue[PackageManager]:
        """Detect which package manager is being used."""
        # Check for poetry
        if "tool" in self.pyproject_data and "poetry" in self.pyproject_data["tool"]:
            # Confirm with poetry.lock
            if (self.project_dir / "poetry.lock").exists():
                return DetectedValue(
                    PackageManager.POETRY, "high", "pyproject.toml [tool.poetry] + poetry.lock"
                )
            return DetectedValue(PackageManager.POETRY, "medium", "pyproject.toml [tool.poetry]")

        # Check for PDM
        if "tool" in self.pyproject_data and "pdm" in self.pyproject_data["tool"]:
            if (self.project_dir / "pdm.lock").exists():
                return DetectedValue(
                    PackageManager.PDM, "high", "pyproject.toml [tool.pdm] + pdm.lock"
                )
            return DetectedValue(PackageManager.PDM, "medium", "pyproject.toml [tool.pdm]")

        # Check for Hatch
        if "tool" in self.pyproject_data and "hatch" in self.pyproject_data["tool"]:
            return DetectedValue(PackageManager.HATCH, "medium", "pyproject.toml [tool.hatch]")

        # Check for Flit
        if "tool" in self.pyproject_data and "flit" in self.pyproject_data["tool"]:
            return DetectedValue(PackageManager.FLIT, "medium", "pyproject.toml [tool.flit]")

        # Check build-backend
        build_system = self.pyproject_data.get("build-system", {})
        build_backend = build_system.get("build-backend", "")

        if "poetry" in build_backend:
            return DetectedValue(PackageManager.POETRY, "medium", "build-backend")
        if "pdm" in build_backend:
            return DetectedValue(PackageManager.PDM, "medium", "build-backend")
        if "hatchling" in build_backend:
            return DetectedValue(PackageManager.HATCH, "medium", "build-backend")
        if "flit" in build_backend:
            return DetectedValue(PackageManager.FLIT, "medium", "build-backend")
        if "setuptools" in build_backend:
            return DetectedValue(PackageManager.SETUPTOOLS, "medium", "build-backend")

        return DetectedValue(PackageManager.UNKNOWN, "low", "no specific markers found")

    def _extract_project_name(self) -> DetectedValue[str] | None:
        """Extract project name from pyproject.toml."""
        # Try poetry section first
        poetry = self.pyproject_data.get("tool", {}).get("poetry", {})
        if "name" in poetry:
            return DetectedValue(poetry["name"], "high", "pyproject.toml [tool.poetry.name]")

        # Try project section (PEP 621)
        project = self.pyproject_data.get("project", {})
        if "name" in project:
            return DetectedValue(project["name"], "high", "pyproject.toml [project.name]")

        # Fall back to directory name
        return DetectedValue(self.project_dir.name, "low", "directory name")

    def _extract_package_name(
        self, project_name: DetectedValue[str] | None
    ) -> DetectedValue[str] | None:
        """Extract or derive package name."""
        # Check for explicit packages in poetry config
        poetry = self.pyproject_data.get("tool", {}).get("poetry", {})
        packages = poetry.get("packages", [])
        if packages and isinstance(packages[0], dict) and "include" in packages[0]:
            return DetectedValue(
                packages[0]["include"], "high", "pyproject.toml [tool.poetry.packages]"
            )

        # Look for packages in src directory
        src_dir = self.project_dir / "src"
        if src_dir.is_dir():
            packages_found = [
                d.name
                for d in src_dir.iterdir()
                if d.is_dir() and not d.name.startswith((".", "__"))
            ]
            if len(packages_found) == 1:
                return DetectedValue(packages_found[0], "high", "src directory structure")
            elif len(packages_found) > 1:
                return DetectedValue(
                    packages_found[0], "medium", "src directory (multiple packages)"
                )

        # Derive from project name
        if project_name:
            derived = project_name.value.replace("-", "_")
            return DetectedValue(
                derived, "medium", f"derived from project name: {project_name.value}"
            )

        return None

    def _extract_version(self) -> DetectedValue[str] | None:
        """Extract version from pyproject.toml."""
        # Try poetry section
        poetry = self.pyproject_data.get("tool", {}).get("poetry", {})
        if "version" in poetry:
            return DetectedValue(poetry["version"], "high", "pyproject.toml [tool.poetry.version]")

        # Try project section
        project = self.pyproject_data.get("project", {})
        if "version" in project:
            return DetectedValue(project["version"], "high", "pyproject.toml [project.version]")

        return DetectedValue("0.1.0", "low", "default value")

    def _extract_description(self) -> DetectedValue[str] | None:
        """Extract description from pyproject.toml."""
        # Try poetry section
        poetry = self.pyproject_data.get("tool", {}).get("poetry", {})
        if "description" in poetry:
            return DetectedValue(
                poetry["description"], "high", "pyproject.toml [tool.poetry.description]"
            )

        # Try project section
        project = self.pyproject_data.get("project", {})
        if "description" in project:
            return DetectedValue(
                project["description"], "high", "pyproject.toml [project.description]"
            )

        return None

    def _extract_python_version(self) -> DetectedValue[str] | None:
        """Extract minimum Python version."""
        # Try poetry section
        poetry = self.pyproject_data.get("tool", {}).get("poetry", {})
        deps = poetry.get("dependencies", {})
        python_spec = deps.get("python", "")

        if python_spec:
            version = self._parse_python_version_spec(python_spec)
            if version:
                return DetectedValue(
                    version, "high", "pyproject.toml [tool.poetry.dependencies.python]"
                )

        # Try project section (PEP 621)
        project = self.pyproject_data.get("project", {})
        requires_python = project.get("requires-python", "")
        if requires_python:
            version = self._parse_python_version_spec(requires_python)
            if version:
                return DetectedValue(version, "high", "pyproject.toml [project.requires-python]")

        # Check ruff target-version
        ruff = self.pyproject_data.get("tool", {}).get("ruff", {})
        target_version = ruff.get("target-version", "")
        if target_version:
            # Convert py311 -> 3.11
            match = re.match(r"py(\d)(\d+)", target_version)
            if match:
                version = f"{match.group(1)}.{match.group(2)}"
                return DetectedValue(version, "medium", "pyproject.toml [tool.ruff.target-version]")

        # Check mypy python_version
        mypy = self.pyproject_data.get("tool", {}).get("mypy", {})
        mypy_version = mypy.get("python_version", "")
        if mypy_version:
            return DetectedValue(
                str(mypy_version), "medium", "pyproject.toml [tool.mypy.python_version]"
            )

        return DetectedValue("3.11", "low", "default value")

    def _parse_python_version_spec(self, spec: str) -> str | None:
        """Parse a Python version specifier to extract the base version."""
        # Handle common patterns: ^3.11, >=3.11, ~3.11, 3.11.*, ==3.11, >=3.11,<4
        patterns = [
            r"[>=^~]*(\d+\.\d+)",  # ^3.11, >=3.11, ~3.11
            r"(\d+\.\d+)\.\*",  # 3.11.*
            r"==(\d+\.\d+)",  # ==3.11
        ]
        for pattern in patterns:
            match = re.search(pattern, spec)
            if match:
                return match.group(1)
        return None

    def _extract_dependencies(self) -> tuple[list[str], list[str]]:
        """Extract main and dev dependencies."""
        main_deps: list[str] = []
        dev_deps: list[str] = []

        # Poetry format
        poetry = self.pyproject_data.get("tool", {}).get("poetry", {})
        if poetry:
            deps = poetry.get("dependencies", {})
            main_deps = [k for k in deps if k != "python"]

            # Dev dependencies can be in various groups
            dev_group = poetry.get("group", {}).get("dev", {}).get("dependencies", {})
            dev_deps = list(dev_group.keys())

            # Also check old-style dev-dependencies
            old_dev = poetry.get("dev-dependencies", {})
            dev_deps.extend(old_dev.keys())

        # PEP 621 format
        project = self.pyproject_data.get("project", {})
        if project:
            deps = project.get("dependencies", [])
            main_deps.extend([self._parse_dep_name(d) for d in deps])

            optional = project.get("optional-dependencies", {})
            for group in ["dev", "test", "development"]:
                if group in optional:
                    dev_deps.extend([self._parse_dep_name(d) for d in optional[group]])

        return list(set(main_deps)), list(set(dev_deps))

    def _parse_dep_name(self, dep_string: str) -> str:
        """Parse dependency name from a requirement string."""
        # Handle: package, package>=1.0, package[extra], package[extra]>=1.0
        match = re.match(r"([a-zA-Z0-9_-]+)", dep_string)
        return match.group(1) if match else dep_string

    def _detect_test_framework(self, dev_deps: list[str]) -> DetectedValue[DetectedTestFramework]:
        """Detect testing framework from dependencies and config."""
        # Check dependencies
        dep_names_lower = [d.lower() for d in dev_deps]

        if "pytest" in dep_names_lower:
            # Confirm with pytest config
            if "tool" in self.pyproject_data and "pytest" in self.pyproject_data["tool"]:
                return DetectedValue(
                    DetectedTestFramework.PYTEST, "high", "dependencies + [tool.pytest]"
                )
            return DetectedValue(DetectedTestFramework.PYTEST, "high", "pytest in dependencies")

        # Check for pytest.ini or conftest.py
        if (self.project_dir / "pytest.ini").exists() or (
            self.project_dir / "conftest.py"
        ).exists():
            return DetectedValue(
                DetectedTestFramework.PYTEST, "medium", "pytest.ini or conftest.py exists"
            )

        if (self.project_dir / "tests" / "conftest.py").exists():
            return DetectedValue(DetectedTestFramework.PYTEST, "medium", "tests/conftest.py exists")

        return DetectedValue(DetectedTestFramework.NONE, "medium", "no test framework detected")

    def _find_existing_tests(self) -> list[Path]:
        """Find existing test files."""
        tests_dir = self.project_dir / "tests"
        if not tests_dir.is_dir():
            return []
        return list(tests_dir.glob("test_*.py"))

    def _detect_linter(self, dev_deps: list[str]) -> DetectedValue[DetectedLinter]:
        """Detect linting/formatting tools."""
        dep_names_lower = [d.lower() for d in dev_deps]

        # Check for ruff
        if (
            "ruff" in dep_names_lower
            or "tool" in self.pyproject_data
            and "ruff" in self.pyproject_data["tool"]
        ):
            if "tool" in self.pyproject_data and "ruff" in self.pyproject_data["tool"]:
                return DetectedValue(DetectedLinter.RUFF, "high", "ruff in config + dependencies")
            return DetectedValue(DetectedLinter.RUFF, "high", "ruff in dependencies")

        # Check for black
        if (
            "black" in dep_names_lower
            or "tool" in self.pyproject_data
            and "black" in self.pyproject_data["tool"]
        ):
            return DetectedValue(DetectedLinter.BLACK, "high", "black in dependencies/config")

        # Check for flake8
        if "flake8" in dep_names_lower:
            return DetectedValue(DetectedLinter.FLAKE8, "medium", "flake8 in dependencies")

        return DetectedValue(DetectedLinter.NONE, "medium", "no linter detected")

    def _detect_type_checker(self, dev_deps: list[str]) -> DetectedValue[DetectedTypeChecker]:
        """Detect type checker."""
        dep_names_lower = [d.lower() for d in dev_deps]

        if (
            "mypy" in dep_names_lower
            or "tool" in self.pyproject_data
            and "mypy" in self.pyproject_data["tool"]
        ):
            if "tool" in self.pyproject_data and "mypy" in self.pyproject_data["tool"]:
                return DetectedValue(
                    DetectedTypeChecker.MYPY, "high", "mypy in config + dependencies"
                )
            return DetectedValue(DetectedTypeChecker.MYPY, "high", "mypy in dependencies")

        if (
            "pyright" in dep_names_lower
            or "tool" in self.pyproject_data
            and "pyright" in self.pyproject_data["tool"]
        ):
            return DetectedValue(
                DetectedTypeChecker.PYRIGHT, "high", "pyright in dependencies/config"
            )

        return DetectedValue(DetectedTypeChecker.NONE, "medium", "no type checker detected")

    def _detect_line_length(self) -> DetectedValue[int] | None:
        """Detect configured line length."""
        # Check ruff
        ruff = self.pyproject_data.get("tool", {}).get("ruff", {})
        if "line-length" in ruff:
            return DetectedValue(
                ruff["line-length"], "high", "pyproject.toml [tool.ruff.line-length]"
            )

        # Check black
        black = self.pyproject_data.get("tool", {}).get("black", {})
        if "line-length" in black:
            return DetectedValue(
                black["line-length"], "high", "pyproject.toml [tool.black.line-length]"
            )

        return DetectedValue(100, "low", "default value")

    def _detect_source_dirs(self) -> list[str]:
        """Detect source directories."""
        dirs: list[str] = []

        if (self.project_dir / "src").is_dir():
            dirs.append("src")

        # Check for top-level package directories
        for item in self.project_dir.iterdir():
            is_package = (
                item.is_dir()
                and not item.name.startswith((".", "_"))
                and (item / "__init__.py").exists()
            )
            if is_package:
                dirs.append(item.name)

        return dirs

    def _determine_missing_fields(self, analysis: ProjectAnalysis) -> list[MissingField]:
        """Determine which fields are missing and need user input."""
        missing: list[MissingField] = []

        if analysis.project_name is None:
            missing.append(
                MissingField(
                    name="project_name",
                    description="Project name",
                    required=True,
                )
            )

        if analysis.python_version is None or analysis.python_version.confidence == "low":
            missing.append(
                MissingField(
                    name="python_version",
                    description="Minimum Python version",
                    required=True,
                    default="3.11",
                    choices=["3.10", "3.11", "3.12", "3.13"],
                )
            )

        if (
            analysis.test_framework is None
            or analysis.test_framework.value == DetectedTestFramework.NONE
        ):
            missing.append(
                MissingField(
                    name="test_framework",
                    description="Testing framework to use",
                    required=False,
                    default="pytest",
                    choices=["pytest", "unittest", "none"],
                )
            )

        if analysis.linter is None or analysis.linter.value == DetectedLinter.NONE:
            missing.append(
                MissingField(
                    name="linter",
                    description="Linting/formatting tool",
                    required=False,
                    default="ruff",
                    choices=["ruff", "black", "none"],
                )
            )

        return missing


def analyze_project(project_dir: Path) -> ProjectAnalysis:
    """Convenience function to analyze a project."""
    analyzer = ProjectAnalyzer(project_dir)
    return analyzer.analyze()
